import os
import json
import time
from typing import List, Dict, Any, Optional
from pydantic import BaseModel, Field

class ResultItem(BaseModel):
    """单条结果记录的数据模型"""
    content: str = Field(..., description="结果内容")
    meta: Dict[str, Any] = Field(default_factory=dict, description="元数据信息")
    time: int = Field(default_factory=lambda: int(time.time()), description="记录时间戳")

    class Config:
        arbitrary_types_allowed = True

class ResultManager:
    """结果管理器，用于维护一个追加写入的jsonl文件"""
    
    def __init__(self, source_dir: Optional[str] = None, event_file: Optional[str] = None):
        """
        初始化结果管理器
        
        Args:
            source_dir: 可选的源目录，如果不提供则使用当前目录
            event_file: 可选的事件文件路径，用于生成结果文件名
        """
        self.source_dir = source_dir or os.getcwd()
        self.result_dir = os.path.join(self.source_dir, ".auto-coder", "results")
        
        if event_file:
            # 获取文件名并去掉后缀
            event_file_name = os.path.splitext(os.path.basename(event_file))[0]
            self.result_file = os.path.join(self.result_dir, f"{event_file_name}.jsonl")
        else:
            self.result_file = os.path.join(self.result_dir, "results.jsonl")
            
        os.makedirs(self.result_dir, exist_ok=True)

    def append(self, content: str, meta: Optional[Dict[str, Any]] = None) -> ResultItem:
        """
        追加一条新的结果记录
        
        Args:
            content: 结果内容
            meta: 可选的元数据信息
            
        Returns:
            ResultItem: 新创建的结果记录
        """
        result_item = ResultItem(
            content=content,
            meta=meta or {},
        )
        
        with open(self.result_file, "a", encoding="utf-8") as f:
            f.write(result_item.model_dump_json() + "\n")
            
        return result_item
    
    def add_result(self, content: str, meta: Optional[Dict[str, Any]] = None) -> ResultItem:
        return self.append(content, meta)
        
    def get_last(self) -> Optional[ResultItem]:
        """
        获取最后一条记录
        
        Returns:
            Optional[ResultItem]: 最后一条记录，如果文件为空则返回None
        """
        if not os.path.exists(self.result_file):
            return None
            
        with open(self.result_file, "r", encoding="utf-8") as f:
            lines = f.readlines()
            if not lines:
                return None
            last_line = lines[-1].strip()
            return ResultItem.model_validate_json(last_line)

    def get_all(self) -> List[ResultItem]:
        """
        获取所有记录
        
        Returns:
            List[ResultItem]: 所有记录的列表
        """
        if not os.path.exists(self.result_file):
            return []
            
        results = []
        with open(self.result_file, "r", encoding="utf-8") as f:
            for line in f:
                line = line.strip()
                if line:  # 跳过空行
                    results.append(ResultItem.model_validate_json(line))
        return results

    def get_by_time_range(self, 
                         start_time: Optional[int] = None, 
                         end_time: Optional[int] = None) -> List[ResultItem]:
        """
        获取指定时间范围内的记录
        
        Args:
            start_time: 开始时间戳
            end_time: 结束时间戳
            
        Returns:
            List[ResultItem]: 符合时间范围的记录列表
        """
        results = []
        for item in self.get_all():
            if start_time and item.time < start_time:
                continue
            if end_time and item.time > end_time:
                continue
            results.append(item)
        return results

    def clear(self) -> None:
        """清空所有记录"""
        if os.path.exists(self.result_file):
            os.remove(self.result_file)


import os
import uuid
from typing import Optional

from autocoder.common.international import get_message
from autocoder.common.ac_style_command_parser import create_config, parse_typed_query
from autocoder.common.ac_style_command_parser.parser import parse_query
from autocoder.auto_coder_runner import (
    get_current_memory,
    get_current_files,
    convert_config_value,
    convert_yaml_config_to_str,
    get_llm_friendly_package_docs,
    get_global_memory_file_paths,
    auto_coder_main,
)
from byzerllm.utils.nontext import Image


class ChatHandler:
    """处理聊天相关的操作"""

    def __init__(self):
        self._config = None

    def _create_config(self):
        """创建 chat 命令的类型化配置"""
        if self._config is None:
            self._config = (
                create_config()
                .collect_remainder("query")  # 收集剩余参数作为聊天查询
                .command("new")
                .max_args(0)  # 开启新会话
                .command("learn")
                .max_args(0)  # 学习模式
                .command("review")
                .max_args(0)  # 审查模式
                .command("no_context")
                .max_args(0)  # 无上下文模式
                .command("copy")
                .max_args(0)  # 复制结果
                .command("save")
                .max_args(0)  # 保存到全局记忆
                .command("mcp")
                .max_args(0)  # MCP 模式
                .command("rag")
                .max_args(0)  # RAG 模式
                .build()
            )
        return self._config

    def handle_chat_command(self, query: str) -> Optional[None]:
        """
        处理聊天命令的主入口

        Args:
            query: 查询字符串

        Returns:
            None: 表示处理了命令，应该返回而不继续执行
        """
        if not query or not query.strip():
            return self._handle_no_args()

        # 检查是否是 /help 命令
        query_stripped = query.strip()
        if query_stripped == "/help":
            self._handle_help_command()
            return None

        # 执行聊天逻辑
        return self._handle_chat(query)

    def _handle_no_args(self) -> None:
        """处理无参数情况"""
        from rich.console import Console
        from rich.panel import Panel

        console = Console()
        console.print(
            Panel(
                "Please provide a chat query or command.",
                title="Chat Command",
                border_style="yellow",
            )
        )
        return None

    def _handle_help_command(self) -> None:
        """处理 /help 命令"""
        help_text = get_message("chat_help_text")
        print(help_text)

    def _handle_chat(self, query: str) -> None:
        """处理聊天逻辑"""
        memory = get_current_memory()
        conf = memory.get("conf", {})

        yaml_config = {
            "include_file": ["./base/base.yml"],
            "include_project_structure": conf.get("include_project_structure", "false")
            in ["true", "True"],
            "human_as_model": conf.get("human_as_model", "false") == "true",
            "skip_build_index": conf.get("skip_build_index", "true") == "true",
            "skip_confirm": conf.get("skip_confirm", "true") == "true",
            "silence": conf.get("silence", "true") == "true",
            "exclude_files": memory.get("exclude_files", []),
        }

        current_files = get_current_files() + get_llm_friendly_package_docs(
            return_paths=True
        )

        if conf.get("enable_global_memory", "false") in ["true", "True", True]:
            current_files += get_global_memory_file_paths()

        yaml_config["urls"] = current_files

        if "emb_model" in conf:
            yaml_config["emb_model"] = conf["emb_model"]

        # 解析命令
        commands_infos = parse_query(query)
        if len(commands_infos) > 0:
            if "query" in commands_infos:
                query = " ".join(commands_infos["query"]["args"])
            else:
                temp_query = ""
                for command, command_info in commands_infos.items():
                    if command_info["args"]:
                        temp_query = " ".join(command_info["args"])
                query = temp_query

        is_new = "new" in commands_infos

        # 特殊处理某些命令
        if "learn" in commands_infos:
            commands_infos["no_context"] = {}

        if "review" in commands_infos:
            commands_infos["no_context"] = {}

        # 转换 OrderedDict 为普通字典以便YAML序列化
        yaml_serializable_commands = {}
        for command, command_info in commands_infos.items():
            if isinstance(command_info, dict):
                yaml_serializable_commands[command] = dict(command_info)
            else:
                yaml_serializable_commands[command] = command_info

        yaml_config["action"] = yaml_serializable_commands

        for key, value in conf.items():
            converted_value = convert_config_value(key, value)
            if converted_value is not None:
                yaml_config[key] = converted_value

        # 处理图片转换
        query = Image.convert_image_paths_from(query)
        yaml_config["query"] = query

        # 保存和执行
        yaml_content = convert_yaml_config_to_str(yaml_config=yaml_config)
        execute_file = os.path.join("actions", f"{uuid.uuid4()}.yml")

        with open(os.path.join(execute_file), "w", encoding="utf-8") as f:
            f.write(yaml_content)

        def execute_ask():
            cmd = ["agent", "chat", "--file", execute_file]
            if is_new:
                cmd.append("--new_session")
            auto_coder_main(cmd)

        try:
            execute_ask()
        finally:
            os.remove(execute_file)

        return None



"""
Messages module for auto-coder international support

This module automatically registers all message files and provides a unified interface.
"""

from .auto_coder_messages import AUTO_CODER_MESSAGES
from .chat_auto_coder_messages import CHAT_AUTO_CODER_MESSAGES
from .sdk_messages import SDK_MESSAGES
from .tool_display_messages import TOOL_DISPLAY_MESSAGES
from .git_helper_plugin_messages import GIT_HELPER_PLUGIN_MESSAGES
from .token_helper_plugin_messages import TOKEN_HELPER_PLUGIN_MESSAGES
from .rules_command_messages import RULES_COMMAND_MESSAGES
from .queue_command_messages import QUEUE_COMMAND_MESSAGES
from .async_command_messages import ASYNC_COMMAND_MESSAGES
from .conversation_command_messages import CONVERSATION_COMMAND_MESSAGES
from .command_help_messages import COMMAND_HELP_MESSAGES
from .workflow_exception_messages import WORKFLOW_EXCEPTION_MESSAGES
from ..message_manager import register_messages


# Automatically register all messages when the module is imported
def register_all_messages():
    """Register all messages with the global message manager"""
    register_messages(AUTO_CODER_MESSAGES)
    register_messages(CHAT_AUTO_CODER_MESSAGES)
    register_messages(SDK_MESSAGES)
    register_messages(TOOL_DISPLAY_MESSAGES)
    register_messages(GIT_HELPER_PLUGIN_MESSAGES)
    register_messages(TOKEN_HELPER_PLUGIN_MESSAGES)
    register_messages(RULES_COMMAND_MESSAGES)
    register_messages(QUEUE_COMMAND_MESSAGES)
    register_messages(ASYNC_COMMAND_MESSAGES)
    register_messages(CONVERSATION_COMMAND_MESSAGES)
    register_messages(COMMAND_HELP_MESSAGES)
    register_messages(WORKFLOW_EXCEPTION_MESSAGES)


# Auto-register on import
register_all_messages()

__all__ = [
    "AUTO_CODER_MESSAGES",
    "CHAT_AUTO_CODER_MESSAGES",
    "SDK_MESSAGES",
    "TOOL_DISPLAY_MESSAGES",
    "GIT_HELPER_PLUGIN_MESSAGES",
    "TOKEN_HELPER_PLUGIN_MESSAGES",
    "RULES_COMMAND_MESSAGES",
    "QUEUE_COMMAND_MESSAGES",
    "ASYNC_COMMAND_MESSAGES",
    "CONVERSATION_COMMAND_MESSAGES",
    "COMMAND_HELP_MESSAGES",
    "WORKFLOW_EXCEPTION_MESSAGES",
    "register_all_messages",
]



import os
import json
import time
from typing import List, Dict, Optional
from pathlib import Path
from filelock import FileLock

from .schema import LLMModel

# 默认内置模型列表
# 价格单位：人民币/百万tokens
DEFAULT_MODELS = [
    {
        "name": "volcengine/deepseek-v3-1-terminus",
        "description": "火山引擎 DeepSeek V3.1 模型，适合编程任务",
        "model_name": "deepseek-v3-1-terminus",
        "model_type": "saas/openai",
        "base_url": "https://ark.cn-beijing.volces.com/api/v3",
        "provider": "volcengine",
        "api_key_path": "",
        "is_reasoning": False,
        "input_price": 4.0,  # 4元/百万tokens (缓存未命中)
        "output_price": 12.0,  # 12元/百万tokens
        "max_output_tokens": 8096,
        "context_window": 128000,
    },
    {
        "name": "volcengine/deepseek-v3-2",
        "description": "火山引擎 DeepSeek V3.2 模型，适合编程任务",
        "model_name": "deepseek-v3-2-251201",
        "model_type": "saas/openai",
        "base_url": "https://ark.cn-beijing.volces.com/api/v3",
        "provider": "volcengine",
        "api_key_path": "",
        "is_reasoning": True,
        "input_price": 2.0,
        "output_price": 3.0,
        "max_output_tokens": 8096,
        "context_window": 128000,
    },
    {
        "name": "volcengine/doubao-seed-1-6-thinking",
        "description": "火山引擎 豆包 Seed-1-6-Thinking 模型，适合编程任务",
        "model_name": "doubao-seed-1-6-thinking-250715",
        "model_type": "saas/openai",
        "base_url": "https://ark.cn-beijing.volces.com/api/v3",
        "provider": "volcengine",
        "api_key_path": "",
        "is_reasoning": True,
        "input_price": 0.8,
        "output_price": 8.0,
        "max_output_tokens": 32000,
        "context_window": 256000,
    },
    {
        "name": "bigmodel/glm-5",
        "description": "智谱 AI GLM-5，国产大模型，性价比高",
        "model_name": "glm-5",
        "model_type": "saas/openai",
        "base_url": "https://open.bigmodel.cn/api/paas/v4/",
        "provider": "bigmodel",
        "api_key_path": "",
        "is_reasoning": False,
        "input_price": 4.0,  # 0.8元/百万tokens
        "output_price": 16.0,  # 2.0元/百万tokens
        "max_output_tokens": 100000,
        "context_window": 200000,
    },
    {
        "name": "bigmodel/coding/glm-5",
        "description": "智谱 AI GLM-5，国产大模型，性价比高,用于coding plan 订阅",
        "model_name": "glm-5",
        "model_type": "saas/openai",
        "base_url": "https://open.bigmodel.cn/api/coding/paas/v4",
        "provider": "bigmodel",
        "api_key_path": "",
        "is_reasoning": False,
        "input_price": 0.0,
        "output_price": 0.0,
        "max_output_tokens": 100000,
        "context_window": 200000,
    },
    {
        "name": "minimax/m2_5",
        "description": "MiniMax M2.5 模型，适合编程任务",
        "model_name": "MiniMax-M2.5",
        "model_type": "saas/openai",
        "base_url": "https://api.minimaxi.com/v1",
        "provider": "minimax",
        "is_reasoning": True,
        "input_price": 2.1,
        "output_price": 8.4,
        "max_output_tokens": 64000,
        "context_window": 204800,
        "api_key_path": "",
    },
    {
        "name": "minimax/coding/m2_5",
        "description": "MiniMax M2.5 模型，用于coding plan 订阅",
        "model_name": "MiniMax-M2.5",
        "model_type": "saas/openai",
        "base_url": "https://api.minimaxi.com/v1",
        "provider": "minimax",
        "is_reasoning": True,
        "input_price": 0.0,
        "output_price": 0.0,
        "max_output_tokens": 64000,
        "context_window": 204800,
        "api_key_path": "",
    },
    {
        "name": "deepseek/deepseek-chat",
        "description": "DeepSeek 官方 API，性能接近 GPT-4，价格低廉",
        "model_name": "deepseek-chat",
        "model_type": "saas/openai",
        "base_url": "https://api.deepseek.com/beta",
        "provider": "deepseek",
        "api_key_path": "",
        "is_reasoning": False,
        "input_price": 2.0,
        "output_price": 3.0,
        "max_output_tokens": 8096,
        "context_window": 128000,
    },
    {
        "name": "deepseek/deepseek-reasoner",
        "description": "DeepSeek 官方 API，推理能力更强，价格低廉",
        "model_name": "deepseek-reasoner",
        "model_type": "saas/openai",
        "base_url": "https://api.deepseek.com/beta",
        "provider": "deepseek",
        "api_key_path": "",
        "is_reasoning": True,
        "input_price": 2.0,
        "output_price": 3.0,
        "max_output_tokens": 64000,
        "context_window": 128000,
        "thinking": {"type": "enabled"},
    },
    {
        "name": "deepseek/deepseek-reasoner-v3.2-speciale",
        "description": "DeepSeek 官方 API，推理能力更强，价格低廉",
        "model_name": "deepseek-reasoner",
        "model_type": "saas/openai",
        "base_url": "https://api.deepseek.com/v3.2_speciale_expires_on_20251215",
        "provider": "deepseek",
        "api_key_path": "",
        "is_reasoning": True,
        "input_price": 2.0,
        "output_price": 3.0,
        "max_output_tokens": 64000,
        "context_window": 128000,
        "thinking": {"type": "enabled"},
    },
    {
        "name": "openrouter/claude-sonnet-4-6",
        "description": "Claude Sonnet 4.6 via OpenRouter，强大的推理能力",
        "model_name": "anthropic/claude-sonnet-4.6",
        "model_type": "saas/openai",
        "base_url": "https://openrouter.ai/api/v1",
        "provider": "openrouter",
        "api_key_path": "",
        "is_reasoning": False,
        "input_price": 21.6,
        "output_price": 108.0,
        "max_output_tokens": 128000,
        "context_window": 1000000,
    },
    {
        "name": "openrouter/claude-opus-4-6",
        "description": "Claude Opus 4.6 via OpenRouter，Anthropic 最强编程模型，适合长流程 Agent 任务",
        "model_name": "anthropic/claude-opus-4.6",
        "model_type": "saas/openai",
        "base_url": "https://openrouter.ai/api/v1",
        "provider": "openrouter",
        "api_key_path": "",
        "is_reasoning": True,
        "input_price": 36.0,  # $5/百万tokens ≈ 36元
        "output_price": 180.0,  # $25/百万tokens ≈ 180元
        "max_output_tokens": 128000,
        "context_window": 1000000,
    },
    {
        "name": "openrouter/openai/gpt-5",
        "description": "GPT-5 via OpenRouter，OpenAI 最新模型",
        "model_name": "openai/gpt-5.2",
        "model_type": "saas/openrouter",
        "base_url": "https://openrouter.ai/api/v1",
        "provider": "openrouter",
        "api_key_path": "",
        "is_reasoning": True,
        "input_price": 9.0,  # 1.25美元/百万tokens ≈ 9元
        "output_price": 72.0,  # 10美元/百万tokens ≈ 72元
        "max_output_tokens": 120000,
        "context_window": 400000,
        "thinking": {"type": "enabled"},
    },
    {
        "name": "openrouter/openai/gpt-5-pro",
        "description": "GPT-5 Pro via OpenRouter，OpenAI 最新模型",
        "model_name": "openai/gpt-5.2-pro",
        "model_type": "saas/openrouter",
        "base_url": "https://openrouter.ai/api/v1",
        "provider": "openrouter",
        "api_key_path": "",
        "is_reasoning": True,
        "input_price": 105.0,
        "output_price": 840.0,
        "max_output_tokens": 12000,
        "context_window": 400000,
        "thinking": {"type": "enabled"},
    },
    {
        "name": "moonshot/kimi-k2.5",
        "description": "Moonshot Kimi K2.5",
        "model_name": "kimi-k2.5",
        "model_type": "saas/openai",
        "base_url": "https://api.moonshot.cn/v1",
        "provider": "moonshot",
        "is_reasoning": True,
        "input_price": 4.0,
        "output_price": 21.0,
        "max_output_tokens": 32000,
        "context_window": 262000,
    },
    {
        "name": "siliconflow/kimi-k2.5",
        "description": "Moonshot Kimi K2.5 via SiliconFlow",
        "model_name": "Pro/moonshotai/Kimi-K2.5",
        "model_type": "saas/openai",
        "base_url": "https://api.siliconflow.cn/v1",
        "provider": "siliconflow",
        "is_reasoning": True,
        "input_price": 4.0,
        "output_price": 21.0,
        "max_output_tokens": 32000,
        "context_window": 262000,
    },
    {
        "name": "siliconflow/deepseek-v3.2",
        "description": "DeepSeek V3.2 via SiliconFlow",
        "model_name": "Pro/deepseek-ai/DeepSeek-V3.2",
        "model_type": "saas/openai",
        "base_url": "https://api.siliconflow.cn/v1",
        "provider": "siliconflow",
        "is_reasoning": True,
        "input_price": 2.0,
        "output_price": 3.0,
        "max_output_tokens": 32000,
        "context_window": 160000,
    },
    {
        "name": "siliconflow/deepseek-v3.1-terminus",
        "description": "DeepSeek V3.1-Terminus via SiliconFlow",
        "model_name": "Pro/deepseek-ai/DeepSeek-V3.1-Terminus",
        "model_type": "saas/openai",
        "base_url": "https://api.siliconflow.cn/v1",
        "provider": "siliconflow",
        "is_reasoning": True,
        "input_price": 4.0,
        "output_price": 12.0,
        "max_output_tokens": 32000,
        "context_window": 160000,
    },
    {
        "name": "siliconflow/glm-5",
        "description": "GLM-5 via SiliconFlow",
        "model_name": "Pro/zai-org/GLM-5",
        "model_type": "saas/openai",
        "base_url": "https://api.siliconflow.cn/v1",
        "provider": "siliconflow",
        "is_reasoning": True,
        "input_price": 18.0,
        "output_price": 22.0,
        "max_output_tokens": 32000,
        "context_window": 200000,
    },
    {
        "name": "siliconflow/minimax-m2.5",
        "description": "MiniMax M2.5 via SiliconFlow",
        "model_name": "Pro/MiniMaxAI/MiniMax-M2.5",
        "model_type": "saas/openai",
        "base_url": "https://api.siliconflow.cn/v1",
        "provider": "siliconflow",
        "is_reasoning": True,
        "input_price": 2.1,
        "output_price": 8.4,
        "max_output_tokens": 64000,
        "context_window": 200000,
        "api_key_path": "",
    },
]


class ModelRegistry:
    """模型注册表，负责模型的持久化和加载（无缓存）"""

    def __init__(self, models_json_path: Optional[str] = None):
        self.models_json_path = Path(
            models_json_path or os.path.expanduser("~/.auto-coder/keys/models.json")
        )

    def _ensure_dir(self):
        """确保目录存在"""
        self.models_json_path.parent.mkdir(parents=True, exist_ok=True)

    def _get_lock_path(self, file_path: Path) -> str:
        """获取锁文件路径"""
        return str(file_path) + ".lock"

    def _load_api_key_from_file(self, model: LLMModel) -> LLMModel:
        """从文件加载 API 密钥"""
        # 设置 base_keys_dir 为配置文件所在目录
        model.base_keys_dir = str(self.models_json_path.parent)

        # 如果 api_key 存在但 api_key_path 不存在，生成 api_key_path
        if model.api_key and not model.api_key_path:
            model.api_key_path = model.name.replace("/", "_")

        # 如果 api_key_path 存在，以它为准读取文件
        if model.api_key_path:
            api_key_file = self.models_json_path.parent / model.api_key_path
            if api_key_file.exists():
                with FileLock(self._get_lock_path(api_key_file), timeout=5):
                    model.api_key = api_key_file.read_text(encoding="utf-8").strip()
        return model

    def load(self) -> List[LLMModel]:
        """每次都重新加载模型列表，合并默认模型和自定义模型"""
        self._ensure_dir()

        # 从默认模型开始
        models_dict = {}
        for model_data in DEFAULT_MODELS:
            # 添加 base_keys_dir 到模型数据
            model_data_with_keys_dir = model_data.copy()
            model_data_with_keys_dir["base_keys_dir"] = str(
                self.models_json_path.parent
            )
            model = LLMModel(**model_data_with_keys_dir)
            models_dict[model.name] = model

        # 如果配置文件存在，读取并合并
        if self.models_json_path.exists():
            try:
                with FileLock(self._get_lock_path(self.models_json_path), timeout=5):
                    with open(self.models_json_path, "r", encoding="utf-8") as f:
                        custom_models_data = json.load(f)

                # 自定义模型会覆盖同名的默认模型
                for model_data in custom_models_data:
                    # 兼容旧版本：添加新字段的默认值
                    if "context_window" not in model_data:
                        model_data["context_window"] = 128 * 1024
                    if "max_output_tokens" not in model_data:
                        model_data["max_output_tokens"] = 8096
                    if "provider" not in model_data:
                        model_data["provider"] = None

                    # 添加 base_keys_dir
                    model_data["base_keys_dir"] = str(self.models_json_path.parent)

                    model = LLMModel(**model_data)
                    models_dict[model.name] = model

            except (json.JSONDecodeError, Exception) as e:
                # JSON 无效时，使用默认模型并重新保存
                print(f"警告：加载 models.json 失败: {e}，使用默认模型")
                self.save(list(models_dict.values()))
        else:
            # 文件不存在，创建默认配置
            self.save(list(models_dict.values()))

        # 加载 API 密钥
        models = []
        for model in models_dict.values():
            models.append(self._load_api_key_from_file(model))

        return models

    def save(self, models: List[LLMModel]) -> None:
        """使用 filelock 保存模型列表到文件"""
        self._ensure_dir()

        # 转换为字典列表，不包含 api_key
        models_data = [model.dict() for model in models]

        # 使用 filelock 写入，超时5秒
        lock_path = self._get_lock_path(self.models_json_path)
        with FileLock(lock_path, timeout=5):
            with open(self.models_json_path, "w", encoding="utf-8") as f:
                json.dump(models_data, f, indent=2, ensure_ascii=False)

    def get(self, name: str) -> Optional[LLMModel]:
        """根据名称获取模型（每次重新加载）"""
        models = self.load()
        for model in models:
            if model.name == name.strip():
                return model
        return None

    def get_all(self) -> Dict[str, LLMModel]:
        """获取所有模型（每次重新加载）"""
        models = self.load()
        return {model.name: model for model in models}

    def add_or_update(self, model: LLMModel) -> None:
        """添加或更新模型"""
        # 先加载所有模型
        models = self.load()

        # 查找是否存在同名模型
        updated = False
        for i, existing_model in enumerate(models):
            if existing_model.name == model.name:
                models[i] = model
                updated = True
                break

        # 如果没有找到同名模型，添加新模型
        if not updated:
            models.append(model)

        # 保存更新后的模型列表
        self.save(models)

    def save_api_key(self, model_name: str, api_key: str) -> bool:
        """保存 API 密钥到文件"""
        model = self.get(model_name)
        if not model:
            return False

        # 确保有 api_key_path
        if not model.api_key_path:
            model.api_key_path = model_name.replace("/", "_")

        # 保存密钥到文件
        api_key_file = self.models_json_path.parent / model.api_key_path

        # 使用 filelock 保存密钥，超时5秒
        lock_path = self._get_lock_path(api_key_file)
        with FileLock(lock_path, timeout=5):
            api_key_file.write_text(api_key.strip(), encoding="utf-8")

        # 更新模型（清除内存中的 api_key）
        model.api_key = None
        self.add_or_update(model)

        return True

    def remove_model(self, model_name: str) -> bool:
        """
        删除模型

        Args:
            model_name: 要删除的模型名称

        Returns:
            bool: 是否成功删除
        """
        # 先检查模型是否存在
        model = self.get(model_name)
        if not model:
            return False

        # 如果是默认模型，不允许删除
        default_model_names = [m["name"] for m in DEFAULT_MODELS]
        if model_name in default_model_names:
            return False

        # 加载所有模型
        models = self.load()

        # 过滤掉要删除的模型
        filtered_models = [m for m in models if m.name != model_name]

        # 如果模型数量没有变化，说明模型不存在于自定义模型中
        if len(filtered_models) == len(models):
            return False

        # 删除 API 密钥文件（如果存在）
        if model.api_key_path:
            api_key_file = self.models_json_path.parent / model.api_key_path
            if api_key_file.exists():
                try:
                    # 使用 filelock 删除密钥文件
                    lock_path = self._get_lock_path(api_key_file)
                    with FileLock(lock_path, timeout=5):
                        api_key_file.unlink()
                except OSError:
                    # 忽略删除密钥文件失败的情况
                    pass

        # 保存更新后的模型列表
        self.save(filtered_models)

        return True



"""
Main conversation manager that integrates all subsystems.

This module provides the PersistConversationManager class, which serves as
the primary interface for conversation and message management, integrating
storage, caching, search, and filtering capabilities.
"""

import time
import uuid
import contextlib
from typing import List, Dict, Any, Optional, Union, Generator, Tuple
from pathlib import Path

from .config import ConversationManagerConfig
from .exceptions import (
    ConversationManagerError,
    ConversationNotFoundError,
    MessageNotFoundError,
    ConcurrencyError,
    DataIntegrityError,
)
from .models import Conversation, ConversationMessage
from .llm_stats_models import LLMCallMetadata
from .file_locker import FileLocker
from .storage import FileStorage, IndexManager
from .cache import CacheManager
from .search import TextSearcher, FilterManager


class PersistConversationManager:
    """
    Main conversation manager integrating all subsystems.

    This class provides a unified interface for conversation and message management,
    with integrated storage, caching, search, and filtering capabilities.
    """

    def __init__(self, config: Optional[ConversationManagerConfig] = None):
        """
        Initialize the conversation manager.

        Args:
            config: Configuration object for the manager
        """
        self.config = config or ConversationManagerConfig()

        # Initialize components
        self._init_storage()
        self._init_cache()
        self._init_search()
        self._init_locks()

        # Statistics tracking
        self._stats = {
            "conversations_created": 0,
            "conversations_loaded": 0,
            "messages_added": 0,
            "cache_hits": 0,
            "cache_misses": 0,
        }

    def _init_storage(self):
        """Initialize storage components."""
        # Ensure storage directory exists
        storage_path = Path(self.config.storage_path)
        storage_path.mkdir(parents=True, exist_ok=True)

        # Initialize storage backend
        self.storage = FileStorage(str(storage_path / "conversations"))

        # Initialize index manager
        self.index_manager = IndexManager(str(storage_path / "index"))

    def _init_cache(self):
        """Initialize cache system."""
        from .cache import MemoryCache

        # Use simple dictionary-based caching for conversations and messages
        self.conversation_cache = MemoryCache(
            max_size=self.config.max_cache_size, default_ttl=self.config.cache_ttl
        )
        self.message_cache = MemoryCache(
            max_size=self.config.max_cache_size
            * 10,  # More messages than conversations
            default_ttl=self.config.cache_ttl,
        )

    def _init_search(self):
        """Initialize search and filtering systems."""
        self.text_searcher = TextSearcher()
        self.filter_manager = FilterManager()

    def _init_locks(self):
        """Initialize locking system."""
        lock_dir = Path(self.config.storage_path) / "locks"
        lock_dir.mkdir(parents=True, exist_ok=True)
        self._lock_dir = str(lock_dir)

    def _get_conversation_lock_file(self, conversation_id: str) -> str:
        """Get lock file path for a conversation."""
        return str(Path(self._lock_dir) / f"{conversation_id}.lock")

    @contextlib.contextmanager
    def _conversation_lock(
        self, conversation_id: str, exclusive: bool = True
    ) -> Generator[None, None, None]:
        """
        Acquire lock for conversation operations.

        Args:
            conversation_id: ID of the conversation to lock
            exclusive: Whether to acquire exclusive (write) lock
        """
        lock_file = self._get_conversation_lock_file(conversation_id)
        locker = FileLocker(lock_file, timeout=self.config.lock_timeout)

        try:
            if exclusive:
                with locker.acquire_write_lock():
                    yield
            else:
                with locker.acquire_read_lock():
                    yield
        except (ConversationNotFoundError, MessageNotFoundError):
            # Re-raise these exceptions as-is (don't wrap in ConcurrencyError)
            raise
        except Exception as e:
            raise ConcurrencyError(
                f"Failed to acquire lock for conversation {conversation_id}: {e}"
            )

    # Conversation Management Methods

    def create_conversation(
        self,
        name: str,
        description: Optional[str] = None,
        initial_messages: Optional[List[Dict[str, Any]]] = None,
        metadata: Optional[Dict[str, Any]] = None,
        conversation_id: Optional[str] = None,
    ) -> str:
        """
        Create a new conversation.

        Args:
            name: Name of the conversation
            description: Optional description
            initial_messages: Optional list of initial messages
            metadata: Optional metadata dictionary
            conversation_id: Optional conversation ID. If not provided, will be auto-generated

        Returns:
            ID of the created conversation

        Raises:
            ConversationManagerError: If conversation creation fails
        """
        try:
            # Create conversation object
            if conversation_id:
                # Check if conversation ID already exists
                if self.storage.conversation_exists(conversation_id):
                    raise ConversationManagerError(
                        f"Conversation with ID {conversation_id} already exists"
                    )

                conversation = Conversation(
                    name=name,
                    description=description,
                    metadata=metadata or {},
                    conversation_id=conversation_id,
                )
            else:
                conversation = Conversation(
                    name=name, description=description, metadata=metadata or {}
                )

            # Add initial messages if provided
            if initial_messages:
                for msg_data in initial_messages:
                    # 支持从 initial_messages 中读取并保留 LLM 元数据
                    llm_md_raw = msg_data.get("llm_metadata")
                    llm_md: Optional[LLMCallMetadata] = None
                    if llm_md_raw is not None:
                        if isinstance(llm_md_raw, LLMCallMetadata):
                            llm_md = llm_md_raw
                        elif isinstance(llm_md_raw, dict):
                            # 从存储字典恢复为强类型对象
                            llm_md = LLMCallMetadata.from_dict(llm_md_raw)

                    message = ConversationMessage(
                        role=msg_data["role"],
                        content=msg_data["content"],
                        metadata=msg_data.get("metadata", {}),
                        llm_metadata=llm_md,
                        reasoning_content=msg_data.get("reasoning_content"),
                    )
                    conversation.add_message(message)

            # Save conversation with locking
            with self._conversation_lock(conversation.conversation_id):
                # Prepare conversation data with message_count
                conversation_data = conversation.to_dict()
                conversation_data["message_count"] = len(conversation.messages)

                # Save to storage
                self.storage.save_conversation(conversation_data)

                # Update index
                self.index_manager.add_conversation(conversation_data)

                # Cache the conversation
                self.conversation_cache.set(
                    conversation.conversation_id, conversation_data
                )

            # Update statistics
            self._stats["conversations_created"] += 1

            return conversation.conversation_id

        except Exception as e:
            raise ConversationManagerError(f"Failed to create conversation: {e}")

    def get_conversation(self, conversation_id: str) -> Optional[Dict[str, Any]]:
        """
        Get a conversation by ID.

        Args:
            conversation_id: ID of the conversation

        Returns:
            Conversation data or None if not found
        """
        try:
            # Try cache first
            cached_conversation = self.conversation_cache.get(conversation_id)
            if cached_conversation:
                self._stats["cache_hits"] += 1
                return cached_conversation

            self._stats["cache_misses"] += 1

            # Load from storage with read lock
            with self._conversation_lock(conversation_id, exclusive=False):
                conversation_data = self.storage.load_conversation(conversation_id)

                if conversation_data:
                    # Cache the loaded conversation
                    self.conversation_cache.set(conversation_id, conversation_data)
                    self._stats["conversations_loaded"] += 1
                    return conversation_data

                return None

        except Exception as e:
            raise ConversationManagerError(
                f"Failed to get conversation {conversation_id}: {e}"
            )

    def update_conversation(
        self,
        conversation_id: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> bool:
        """
        Update conversation metadata.

        Args:
            conversation_id: ID of the conversation to update
            name: New name (optional)
            description: New description (optional)
            metadata: New metadata (optional)

        Returns:
            True if update was successful

        Raises:
            ConversationNotFoundError: If conversation doesn't exist
        """
        try:
            with self._conversation_lock(conversation_id):
                # Load current conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)

                # Create conversation object and update fields
                conversation = Conversation.from_dict(conversation_data)

                if name is not None:
                    conversation.name = name
                if description is not None:
                    conversation.description = description
                if metadata is not None:
                    if conversation.metadata is None:
                        conversation.metadata = {}
                    conversation.metadata.update(metadata)

                conversation.updated_at = time.time()

                # Save updated conversation with message_count
                updated_data = conversation.to_dict()
                updated_data["message_count"] = len(conversation.messages)
                self.storage.save_conversation(updated_data)

                # Update index
                self.index_manager.update_conversation(updated_data)

                # Update cache
                self.conversation_cache.set(conversation_id, updated_data)

                return True

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to update conversation {conversation_id}: {e}"
            )

    def delete_conversation(self, conversation_id: str) -> bool:
        """
        Delete a conversation.

        Args:
            conversation_id: ID of the conversation to delete

        Returns:
            True if deletion was successful
        """
        try:
            with self._conversation_lock(conversation_id):
                # Check if conversation exists
                if not self.storage.conversation_exists(conversation_id):
                    return False

                # Delete from storage
                self.storage.delete_conversation(conversation_id)

                # Remove from index
                self.index_manager.remove_conversation(conversation_id)

                # Remove from cache
                self.conversation_cache.delete(conversation_id)

                return True

        except Exception as e:
            raise ConversationManagerError(
                f"Failed to delete conversation {conversation_id}: {e}"
            )

    def copy_conversation(
        self,
        source_conversation_id: str,
        new_name: Optional[str] = None,
        new_description: Optional[str] = None,
        new_conversation_id: Optional[str] = None,
        copy_messages: bool = True,
        copy_metadata: bool = True,
    ) -> str:
        """
        复制一个会话，生成新的会话ID。

        Args:
            source_conversation_id: 源会话ID
            new_name: 新会话名称，如果为None则使用 "源会话名称 (副本)"
            new_description: 新会话描述，如果为None则复制源会话描述
            new_conversation_id: 可选的新会话ID，如果为None则自动生成
            copy_messages: 是否复制消息，默认True
            copy_metadata: 是否复制元数据，默认True

        Returns:
            新会话的ID

        Raises:
            ConversationNotFoundError: 如果源会话不存在
            ConversationManagerError: 如果复制失败或新会话ID已存在
        """
        try:
            # 获取源会话数据
            source_conversation = self.get_conversation(source_conversation_id)
            if not source_conversation:
                raise ConversationNotFoundError(source_conversation_id)

            # 确定新会话名称
            if new_name is None:
                source_name = source_conversation.get("name", "未命名会话")
                new_name = f"{source_name} (副本)"

            # 确定新会话描述
            if new_description is None:
                new_description = source_conversation.get("description")

            # 准备元数据
            new_metadata = None
            if copy_metadata:
                source_metadata = source_conversation.get("metadata")
                if source_metadata:
                    # 深拷贝元数据
                    import copy

                    new_metadata = copy.deepcopy(source_metadata)

            # 准备初始消息
            initial_messages = None
            if copy_messages:
                source_messages = source_conversation.get("messages", [])
                if source_messages:
                    # 复制消息内容，但不复制消息ID（会自动生成新的）
                    initial_messages = []
                    for msg in source_messages:
                        msg_copy = {
                            "role": msg.get("role"),
                            "content": msg.get("content"),
                        }
                        # 复制消息元数据
                        if msg.get("metadata"):
                            import copy

                            msg_copy["metadata"] = copy.deepcopy(msg.get("metadata"))
                        # 复制 LLM 元数据
                        if msg.get("llm_metadata"):
                            import copy

                            msg_copy["llm_metadata"] = copy.deepcopy(
                                msg.get("llm_metadata")
                            )
                        initial_messages.append(msg_copy)

            # 创建新会话
            new_id = self.create_conversation(
                name=new_name,
                description=new_description,
                initial_messages=initial_messages,
                metadata=new_metadata,
                conversation_id=new_conversation_id,
            )

            return new_id

        except ConversationNotFoundError:
            raise
        except ConversationManagerError:
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to copy conversation {source_conversation_id}: {e}"
            )

    def list_conversations(
        self,
        limit: Optional[int] = None,
        offset: int = 0,
        filters: Optional[Dict[str, Any]] = None,
        sort_by: str = "updated_at",
        sort_desc: bool = True,
    ) -> List[Dict[str, Any]]:
        """
        List conversations with optional filtering and sorting.

        Args:
            limit: Maximum number of conversations to return
            offset: Number of conversations to skip
            filters: Optional filter criteria
            sort_by: Field to sort by
            sort_desc: Whether to sort in descending order

        Returns:
            List of conversation data
        """
        try:
            # Convert sort_desc boolean to sort_order string
            sort_order = "desc" if sort_desc else "asc"

            # Get conversations from index with sorting and pagination
            conversations = self.index_manager.list_conversations(
                limit=limit, offset=offset, sort_by=sort_by, sort_order=sort_order
            )

            # Apply filters if provided
            if filters:
                conversations = self.filter_manager.apply_filters(
                    conversations, filters
                )

            return conversations

        except Exception as e:
            raise ConversationManagerError(f"Failed to list conversations: {e}")

    # Message Management Methods

    def append_message(
        self,
        conversation_id: str,
        role: str,
        content: Union[str, Dict[str, Any], List[Any]],
        metadata: Optional[Dict[str, Any]] = None,
        llm_metadata: Optional["LLMCallMetadata"] = None,
        reasoning_content: Optional[str] = None,
    ) -> str:
        """
        Append a message to a conversation.

        Args:
            conversation_id: ID of the conversation
            role: Role of the message sender
            content: Message content
            metadata: Optional message metadata
            llm_metadata: Optional LLM call metadata (tokens, cache info, etc.)
            reasoning_content: Optional reasoning content (e.g., from deepseek models)

        Returns:
            ID of the added message

        Raises:
            ConversationNotFoundError: If conversation doesn't exist
        """
        try:
            # Create message object
            message = ConversationMessage(
                role=role,
                content=content,
                metadata=metadata or {},
                llm_metadata=llm_metadata,
                reasoning_content=reasoning_content,
            )

            with self._conversation_lock(conversation_id):
                # Load conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)

                # Add message to conversation
                conversation = Conversation.from_dict(conversation_data)
                conversation.add_message(message)

                # Save updated conversation with message_count
                updated_data = conversation.to_dict()
                updated_data["message_count"] = len(conversation.messages)
                self.storage.save_conversation(updated_data)

                # Update index
                self.index_manager.update_conversation(updated_data)

                # Update cache
                self.conversation_cache.set(conversation_id, updated_data)
                self.message_cache.set(
                    f"{conversation_id}:{message.message_id}", message.to_dict()
                )

            # Update statistics
            self._stats["messages_added"] += 1

            return message.message_id

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to append message to conversation {conversation_id}: {e}"
            )

    def append_messages(
        self, conversation_id: str, messages: List[Dict[str, Any]]
    ) -> List[str]:
        """
        Append multiple messages to a conversation.

        Args:
            conversation_id: ID of the conversation
            messages: List of message data dictionaries. Each dictionary can contain:
                - role: Message role (required)
                - content: Message content (required)
                - metadata: Message metadata (optional)
                - llm_metadata: LLM call metadata (optional)
                - reasoning_content: Reasoning content (optional)

        Returns:
            List of message IDs
        """
        message_ids = []

        for msg_data in messages:
            message_id = self.append_message(
                conversation_id,
                msg_data["role"],
                msg_data["content"],
                msg_data.get("metadata"),
                msg_data.get("llm_metadata"),
                msg_data.get("reasoning_content"),
            )
            message_ids.append(message_id)

        return message_ids

    def get_messages(
        self,
        conversation_id: str,
        limit: Optional[int] = None,
        offset: int = 0,
        message_ids: Optional[List[str]] = None,
    ) -> List[Dict[str, Any]]:
        """
        Get messages from a conversation.

        Args:
            conversation_id: ID of the conversation
            limit: Maximum number of messages to return
            offset: Number of messages to skip
            message_ids: Optional list of specific message IDs

        Returns:
            List of message data
        """
        try:
            # Get conversation
            conversation_data = self.get_conversation(conversation_id)
            if not conversation_data:
                raise ConversationNotFoundError(conversation_id)

            messages = conversation_data.get("messages", [])

            # Filter by message IDs if provided
            if message_ids:
                id_set = set(message_ids)
                messages = [msg for msg in messages if msg.get("message_id") in id_set]

            # Apply pagination
            end_idx = offset + limit if limit else None
            return messages[offset:end_idx]

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to get messages from conversation {conversation_id}: {e}"
            )

    def get_message(
        self, conversation_id: str, message_id: str
    ) -> Optional[Dict[str, Any]]:
        """
        Get a specific message.

        Args:
            conversation_id: ID of the conversation
            message_id: ID of the message

        Returns:
            Message data or None if not found
        """
        try:
            # Try cache first
            cached_message = self.message_cache.get(f"{conversation_id}:{message_id}")
            if cached_message:
                return cached_message

            # Get from conversation
            conversation_data = self.get_conversation(conversation_id)
            if not conversation_data:
                return None

            # Find message
            for message in conversation_data.get("messages", []):
                if message.get("message_id") == message_id:
                    # Cache the message
                    self.message_cache.set(f"{conversation_id}:{message_id}", message)
                    return message

            return None

        except Exception as e:
            raise ConversationManagerError(f"Failed to get message {message_id}: {e}")

    def get_message_count(self, conversation_id: str) -> int:
        """
        Get the number of messages in a conversation.

        Args:
            conversation_id: ID of the conversation

        Returns:
            Number of messages in the conversation

        Raises:
            ConversationNotFoundError: If conversation doesn't exist
        """
        try:
            # Try cache first
            cached_conversation = self.conversation_cache.get(conversation_id)
            if cached_conversation:
                return len(cached_conversation.get("messages", []))

            # Try index for message count
            index_data = self.index_manager.get_conversation(conversation_id)
            if index_data:
                # Check if message_count is available in index
                if "message_count" in index_data and isinstance(
                    index_data["message_count"], int
                ):
                    return index_data["message_count"]
                # Check if messages are included in index (for backward compatibility)
                if "messages" in index_data:
                    return len(index_data.get("messages", []))

            # Fallback: load full conversation
            conversation_data = self.get_conversation(conversation_id)
            if not conversation_data:
                raise ConversationNotFoundError(conversation_id)

            return len(conversation_data.get("messages", []))

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to get message count for conversation {conversation_id}: {e}"
            )

    def update_message(
        self,
        conversation_id: str,
        message_id: str,
        content: Optional[Union[str, Dict[str, Any], List[Any]]] = None,
        metadata: Optional[Dict[str, Any]] = None,
        llm_metadata: Optional["LLMCallMetadata"] = None,
        reasoning_content: Optional[str] = None,
    ) -> bool:
        """
        Update a message.

        Args:
            conversation_id: ID of the conversation
            message_id: ID of the message to update
            content: New content (optional)
            metadata: New metadata (optional)
            llm_metadata: LLM call metadata (optional)
            reasoning_content: Reasoning content (optional)

        Returns:
            True if update was successful
        """
        try:
            with self._conversation_lock(conversation_id):
                # Load conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)

                conversation = Conversation.from_dict(conversation_data)

                # Find and update message
                for i, message_data in enumerate(conversation.messages):
                    msg = ConversationMessage.from_dict(message_data)
                    if msg.message_id == message_id:
                        # Update message fields
                        if content is not None:
                            msg.content = content
                        if metadata is not None:
                            if msg.metadata is None:
                                msg.metadata = {}
                            msg.metadata.update(metadata)
                        if llm_metadata is not None:
                            msg.llm_metadata = llm_metadata
                        if reasoning_content is not None:
                            msg.reasoning_content = reasoning_content

                        # Update timestamp
                        msg.timestamp = time.time()

                        # Replace in conversation
                        conversation.messages[i] = msg.to_dict()
                        conversation.updated_at = time.time()

                        # Save updated conversation with message_count
                        updated_data = conversation.to_dict()
                        updated_data["message_count"] = len(conversation.messages)
                        self.storage.save_conversation(updated_data)

                        # Update index
                        self.index_manager.update_conversation(updated_data)

                        # Update caches
                        self.conversation_cache.set(conversation_id, updated_data)
                        self.message_cache.set(
                            f"{conversation_id}:{message_id}", msg.to_dict()
                        )

                        return True

                raise MessageNotFoundError(message_id)

        except (ConversationNotFoundError, MessageNotFoundError):
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to update message {message_id}: {e}"
            )

    def delete_message(self, conversation_id: str, message_id: str) -> bool:
        """
        Delete a message from a conversation.

        Args:
            conversation_id: ID of the conversation
            message_id: ID of the message to delete

        Returns:
            True if deletion was successful
        """
        try:
            with self._conversation_lock(conversation_id):
                # Load conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)

                conversation = Conversation.from_dict(conversation_data)

                # Find and remove message
                original_count = len(conversation.messages)
                conversation.messages = [
                    msg
                    for msg in conversation.messages
                    if msg.get("message_id") != message_id
                ]

                if len(conversation.messages) == original_count:
                    raise MessageNotFoundError(message_id)

                conversation.updated_at = time.time()

                # Save updated conversation with message_count
                updated_data = conversation.to_dict()
                updated_data["message_count"] = len(conversation.messages)
                self.storage.save_conversation(updated_data)

                # Update index
                self.index_manager.update_conversation(updated_data)

                # Update cache
                self.conversation_cache.set(conversation_id, updated_data)
                self.message_cache.delete(f"{conversation_id}:{message_id}")

                return True

        except (ConversationNotFoundError, MessageNotFoundError):
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to delete message {message_id}: {e}"
            )

    def delete_message_pair(self, conversation_id: str, user_message_id: str) -> bool:
        """
        Delete a user message and its corresponding assistant reply.

        Args:
            conversation_id: ID of the conversation
            user_message_id: ID of the user message

        Returns:
            True if deletion was successful
        """
        try:
            with self._conversation_lock(conversation_id):
                # Load conversation
                conversation_data = self.storage.load_conversation(conversation_id)
                if not conversation_data:
                    raise ConversationNotFoundError(conversation_id)

                conversation = Conversation.from_dict(conversation_data)

                # Find user message and next assistant message
                messages_to_remove = []
                for i, message in enumerate(conversation.messages):
                    if message.get("message_id") == user_message_id:
                        messages_to_remove.append(i)
                        # Check if next message is assistant reply
                        if (
                            i + 1 < len(conversation.messages)
                            and conversation.messages[i + 1].get("role") == "assistant"
                        ):
                            messages_to_remove.append(i + 1)
                        break

                if not messages_to_remove:
                    raise MessageNotFoundError(user_message_id)

                # Remove messages (in reverse order to maintain indices)
                assistant_message_id = None
                for idx in reversed(messages_to_remove):
                    if idx == messages_to_remove[-1] and len(messages_to_remove) > 1:
                        assistant_message_id = conversation.messages[idx].get(
                            "message_id"
                        )
                    del conversation.messages[idx]

                conversation.updated_at = time.time()

                # Save updated conversation with message_count
                updated_data = conversation.to_dict()
                updated_data["message_count"] = len(conversation.messages)
                self.storage.save_conversation(updated_data)

                # Update index
                self.index_manager.update_conversation(updated_data)

                # Update caches
                self.conversation_cache.set(conversation_id, updated_data)
                self.message_cache.delete(f"{conversation_id}:{user_message_id}")
                if assistant_message_id:
                    self.message_cache.delete(
                        f"{conversation_id}:{assistant_message_id}"
                    )

                return True

        except (ConversationNotFoundError, MessageNotFoundError):
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to delete message pair {user_message_id}: {e}"
            )

    # Search and Filter Methods

    def search_conversations(
        self,
        query: str,
        search_in_messages: bool = True,
        filters: Optional[Dict[str, Any]] = None,
        max_results: Optional[int] = None,
        min_score: float = 0.0,
    ) -> List[Dict[str, Any]]:
        """
        Search conversations.

        Args:
            query: Search query
            search_in_messages: Whether to search in message content
            filters: Optional filter criteria
            max_results: Maximum number of results
            min_score: Minimum relevance score

        Returns:
            List of matching conversations with scores
        """
        try:
            # Get all conversations
            conversations = self.index_manager.list_conversations()

            # Apply filters first if provided
            if filters:
                conversations = self.filter_manager.apply_filters(
                    conversations, filters
                )

            # If search_in_messages is True, load full conversation data
            if search_in_messages:
                full_conversations = []
                for conv in conversations:
                    conv_id = conv.get("conversation_id")
                    if conv_id:
                        full_conv = self.get_conversation(conv_id)
                        if full_conv:
                            full_conversations.append(full_conv)
                conversations = full_conversations

            # Perform text search
            results = self.text_searcher.search_conversations(
                query, conversations, max_results, min_score
            )

            return [{"conversation": conv, "score": score} for conv, score in results]

        except Exception as e:
            raise ConversationManagerError(f"Failed to search conversations: {e}")

    def search_messages(
        self,
        conversation_id: str,
        query: str,
        filters: Optional[Dict[str, Any]] = None,
        max_results: Optional[int] = None,
        min_score: float = 0.0,
    ) -> List[Dict[str, Any]]:
        """
        Search messages in a conversation.

        Args:
            conversation_id: ID of the conversation
            query: Search query
            filters: Optional filter criteria
            max_results: Maximum number of results
            min_score: Minimum relevance score

        Returns:
            List of matching messages with scores
        """
        try:
            # Get conversation messages
            messages = self.get_messages(conversation_id)

            # Apply filters if provided
            if filters:
                messages = self.filter_manager.apply_filters(messages, filters)

            # Perform text search
            results = self.text_searcher.search_messages(
                query, messages, max_results, min_score
            )

            return [{"message": msg, "score": score} for msg, score in results]

        except Exception as e:
            raise ConversationManagerError(f"Failed to search messages: {e}")

    # Utility and Management Methods

    def get_statistics(self) -> Dict[str, Any]:
        """
        Get manager statistics.

        Returns:
            Dictionary with statistics
        """
        cache_stats = {
            "conversation_cache_size": self.conversation_cache.size(),
            "message_cache_size": self.message_cache.size(),
        }

        return {
            **self._stats,
            "cache_stats": cache_stats,
            "total_conversations": len(self.index_manager.list_conversations()),
            "current_conversation_id": self.get_current_conversation_id(),
            "all_current_conversations": self.get_all_current_conversations(),
            "namespaces": self.list_namespaces(),
            "default_namespace": self.config.default_namespace,
            "storage_path": self.config.storage_path,
        }

    def health_check(self) -> Dict[str, Any]:
        """
        Perform health check of all components.

        Returns:
            Health status dictionary
        """
        health_status = {
            "status": "healthy",
            "storage": True,
            "cache": True,
            "index": True,
            "search": True,
            "issues": [],
        }

        try:
            # Check storage
            if not Path(self.config.storage_path).exists():
                health_status["storage"] = False
                health_status["issues"].append("Storage directory not accessible")

            # Check cache
            conv_cache_size = self.conversation_cache.size()
            if conv_cache_size > self.config.max_cache_size:
                health_status["issues"].append("Conversation cache size exceeds limit")

            # Check index consistency
            try:
                conversations = self.index_manager.list_conversations()
                health_status["total_conversations"] = len(conversations)
            except Exception as e:
                health_status["index"] = False
                health_status["issues"].append(f"Index error: {e}")

            # Determine overall status
            if not all(
                [
                    health_status["storage"],
                    health_status["cache"],
                    health_status["index"],
                    health_status["search"],
                ]
            ):
                health_status["status"] = "degraded"

            if health_status["issues"]:
                health_status["status"] = (
                    "warning"
                    if health_status["status"] == "healthy"
                    else health_status["status"]
                )

        except Exception as e:
            health_status["status"] = "unhealthy"
            health_status["issues"].append(f"Health check failed: {e}")

        return health_status

    @contextlib.contextmanager
    def transaction(self, conversation_id: str) -> Generator[None, None, None]:
        """
        Transaction context manager for atomic operations.

        Args:
            conversation_id: ID of the conversation for the transaction
        """
        with self._conversation_lock(conversation_id):
            try:
                yield
            except Exception:
                # In a full implementation, we would rollback changes here
                # For now, we just re-raise the exception
                raise

    def clear_cache(self):
        """Clear all caches."""
        self.conversation_cache.clear()
        self.message_cache.clear()

    def rebuild_index(self):
        """Rebuild the conversation index from storage."""
        try:
            # Clear existing index
            self.index_manager._index_data.clear()

            # Load all conversations from storage and rebuild index
            conversations = self.storage.list_conversations()

            for conversation_data in conversations:
                if conversation_data and conversation_data.get("conversation_id"):
                    self.index_manager.add_conversation(conversation_data)

        except Exception as e:
            raise ConversationManagerError(f"Failed to rebuild index: {e}")

    def close(self):
        """Clean up resources."""
        # Clear caches
        self.clear_cache()

        # Save any pending index changes
        try:
            self.index_manager._save_index()
        except Exception:
            pass  # Ignore errors during cleanup

    # Current Conversation Management Methods

    def set_current_conversation(
        self, conversation_id: str, namespace: Optional[str] = None
    ) -> bool:
        """
        设置当前对话。

        Args:
            conversation_id: 要设置为当前对话的ID
            namespace: 命名空间，None表示使用默认命名空间

        Returns:
            True if setting was successful

        Raises:
            ConversationNotFoundError: 如果对话不存在
        """
        try:
            # 如果没有指定命名空间，使用配置中的默认命名空间
            if namespace is None:
                namespace = self.config.default_namespace

            # 验证对话是否存在
            conversation_data = self.get_conversation(conversation_id)
            if not conversation_data:
                raise ConversationNotFoundError(conversation_id)

            # 设置当前对话
            success = self.index_manager.set_current_conversation(
                conversation_id, namespace
            )
            if not success:
                raise ConversationManagerError(
                    f"Failed to set current conversation: {conversation_id}"
                )

            return True

        except ConversationNotFoundError:
            raise
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to set current conversation {conversation_id}: {e}"
            )

    def get_current_conversation_id(
        self, namespace: Optional[str] = None
    ) -> Optional[str]:
        """
        获取当前对话ID。

        Args:
            namespace: 命名空间，None表示使用默认命名空间

        Returns:
            当前对话ID，如果未设置返回None
        """
        try:
            # 如果没有指定命名空间，使用配置中的默认命名空间
            if namespace is None:
                namespace = self.config.default_namespace

            return self.index_manager.get_current_conversation_id(namespace)
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to get current conversation ID: {e}"
            )

    def get_current_conversation(
        self, namespace: Optional[str] = None
    ) -> Optional[Dict[str, Any]]:
        """
        获取当前对话的完整数据。

        Args:
            namespace: 命名空间，None表示使用默认命名空间

        Returns:
            当前对话的数据字典，如果未设置或对话不存在返回None
        """
        try:
            current_id = self.get_current_conversation_id(namespace)
            if not current_id:
                return None

            return self.get_conversation(current_id)

        except Exception as e:
            raise ConversationManagerError(f"Failed to get current conversation: {e}")

    def clear_current_conversation(self, namespace: Optional[str] = None) -> bool:
        """
        清除当前对话设置。

        Args:
            namespace: 命名空间，None表示使用默认命名空间

        Returns:
            True if clearing was successful
        """
        try:
            # 如果没有指定命名空间，使用配置中的默认命名空间
            if namespace is None:
                namespace = self.config.default_namespace

            success = self.index_manager.clear_current_conversation(namespace)
            if not success:
                raise ConversationManagerError("Failed to clear current conversation")

            return True

        except Exception as e:
            raise ConversationManagerError(f"Failed to clear current conversation: {e}")

    def append_message_to_current(
        self,
        role: str,
        content: Union[str, Dict[str, Any], List[Any]],
        metadata: Optional[Dict[str, Any]] = None,
        llm_metadata: Optional[LLMCallMetadata] = None,
        namespace: Optional[str] = None,
        reasoning_content: Optional[str] = None,
    ) -> str:
        """
        向当前对话添加消息。

        Args:
            role: 消息角色
            content: 消息内容
            metadata: 可选的消息元数据
            llm_metadata: 可选的 LLM 调用元数据
            namespace: 命名空间，None表示使用默认命名空间
            reasoning_content: 可选的推理内容（如 deepseek 模型的推理过程）

        Returns:
            消息ID

        Raises:
            ConversationManagerError: 如果没有设置当前对话或添加失败
        """
        try:
            current_id = self.get_current_conversation_id(namespace)
            if not current_id:
                ns_desc = (
                    f"namespace '{namespace}'" if namespace else "default namespace"
                )
                raise ConversationManagerError(
                    f"No current conversation set for {ns_desc}"
                )

            return self.append_message(
                current_id, role, content, metadata, llm_metadata, reasoning_content
            )

        except Exception as e:
            raise ConversationManagerError(
                f"Failed to append message to current conversation: {e}"
            )

    def list_namespaces(self) -> List[Optional[str]]:
        """
        列出所有已使用的命名空间。

        Returns:
            命名空间列表，None 表示默认命名空间
        """
        try:
            return self.index_manager.list_namespaces()
        except Exception as e:
            raise ConversationManagerError(f"Failed to list namespaces: {e}")

    def get_all_current_conversations(self) -> Dict[Optional[str], str]:
        """
        获取所有命名空间的当前对话ID。

        Returns:
            命名空间到对话ID的映射
        """
        try:
            return self.index_manager.get_all_current_conversations()
        except Exception as e:
            raise ConversationManagerError(
                f"Failed to get all current conversations: {e}"
            )



"""
Command help messages for internationalization
Contains all help text messages for various commands

使用 byzerllm prompt function 风格定义帮助文本，方便编辑和维护
"""

import byzerllm


# /conf command help text definitions
@byzerllm.prompt()
def conf_help_en() -> str:
    """
    /conf command usage:
      /conf [pattern]    - Show configurations (merged view). Optional wildcard pattern (e.g., *_model, api*).
      /conf <key>:<value> - Set or update a configuration key (shortcut for /conf /set).
      /conf /global <key>:<value> - Set or update a global configuration key (shortcut).
      /conf /get <key>    - Get the value of a specific configuration key (merged view).
      /conf /set <key>:<value> - Set or update a configuration key (project scope).
                           Value parsed (bool, int, float, None) or treated as string.
                           Use quotes ("value with spaces") for explicit strings.
      /conf /drop <key> - Delete a configuration key (project scope).
      /conf /delete <key> - Delete a configuration key (project scope).
      /conf /export <path> - Export current configuration to a file.
      /conf /import <path> - Import configuration from a file.
      /conf /help        - Show this help message.

    Global configuration (stored in ~/.auto-coder):
      /conf /global /list [pattern] - List global configurations.
      /conf /global /get <key>      - Get a global configuration value.
      /conf /global /set <key> <value> - Set a global configuration.
      (Shorthand also supported: /conf /global <key>:<value>)
      /conf /global /delete <key>   - Delete a global configuration.

    Note: When reading configs, project values take precedence over global values.
    """


@byzerllm.prompt()
def conf_help_zh() -> str:
    """
    /conf 命令用法:
      /conf [pattern]    - 显示配置（合并视图）。可选通配符模式 (例如: *_model, api*).
      /conf <key>:<value> - 设置或更新配置项 (/conf /set 的快捷方式，项目级).
      /conf /global <key>:<value> - 设置或更新全局配置项（简写形式）.
      /conf /get <key>    - 获取指定配置项的值（合并视图）.
      /conf /set <key>:<value> - 设置或更新配置项（项目级）.
                           值会被解析为 (bool, int, float, None) 或字符串.
                           使用引号 ("带空格的值") 表示明确的字符串.
      /conf /drop <key> - 删除配置项（项目级）.
      /conf /delete <key> - 删除配置项（项目级）.
      /conf /export <path> - 将当前配置导出到文件.
      /conf /import <path> - 从文件导入配置.
      /conf /help        - 显示此帮助信息.

    全局配置（存储在 ~/.auto-coder）:
      /conf /global /list [pattern] - 列出全局配置.
      /conf /global /get <key>      - 获取全局配置值.
      /conf /global /set <key> <value> - 设置全局配置.
      （同样支持简写：/conf /global <key>:<value>）
      /conf /global /delete <key>   - 删除全局配置.

    注意：读取配置时，项目级配置优先于全局配置。
    """


@byzerllm.prompt()
def conf_help_ja() -> str:
    """
    /conf コマンド使用法:
      /conf [pattern]    - 設定を表示. オプションのワイルドカードパターン (例: *_model, api*).
      /conf <key>:<value> - 設定キーを設定または更新 (/conf /set のショートカット).
      /conf /get <key>    - 特定の設定キーの値を取得.
      /conf /set <key>:<value> - 設定キーを設定または更新.
                           値は (bool, int, float, None) として解析されるか文字列として扱われます.
                           明示的な文字列には引用符 ("スペースを含む値") を使用.
      /conf /drop <key> - 設定キーを削除.
      /conf /delete <key> - 設定キーを削除.
      /conf /export <path> - 現在の設定をファイルにエクスポート.
      /conf /import <path> - ファイルから設定をインポート.
      /conf /help        - このヘルプメッセージを表示.
    """


@byzerllm.prompt()
def conf_help_ar() -> str:
    """
    استخدام أمر /conf:
      /conf [pattern]    - إظهار التكوينات. نمط أحرف البدل الاختياري (مثل: *_model, api*).
      /conf <key>:<value> - تعيين أو تحديث مفتاح التكوين (اختصار لـ /conf /set).
      /conf /get <key>    - الحصول على قيمة مفتاح تكوين محدد.
      /conf /set <key>:<value> - تعيين أو تحديث مفتاح التكوين.
                           يتم تحليل القيمة كـ (bool, int, float, None) أو التعامل معها كسلسلة نصية.
                           استخدم علامات الاقتباس ("قيمة بمسافات") للسلاسل النصية الصريحة.
      /conf /drop <key> - حذف مفتاح التكوين.
      /conf /delete <key> - حذف مفتاح التكوين.
      /conf /export <path> - تصدير التكوين الحالي إلى ملف.
      /conf /import <path> - استيراد التكوين من ملف.
      /conf /help        - إظهار رسالة المساعدة هذه.
    """


@byzerllm.prompt()
def conf_help_ru() -> str:
    """
    Использование команды /conf:
      /conf [pattern]    - Показать конфигурации. Необязательный шаблон с подстановочными знаками (например: *_model, api*).
      /conf <key>:<value> - Установить или обновить ключ конфигурации (сокращение для /conf /set).
      /conf /get <key>    - Получить значение определенного ключа конфигурации.
      /conf /set <key>:<value> - Установить или обновить ключ конфигурации.
                           Значение парсится как (bool, int, float, None) или обрабатывается как строка.
                           Используйте кавычки ("значение с пробелами") для явных строк.
      /conf /drop <key> - Удалить ключ конфигурации.
      /conf /delete <key> - Удалить ключ конфигурации.
      /conf /export <path> - Экспортировать текущую конфигурацию в файл.
      /conf /import <path> - Импортировать конфигурацию из файла.
      /conf /help        - Показать это сообщение справки.
    """


# /auto command help text definitions
@byzerllm.prompt()
def auto_help_en() -> str:
    """
    /auto command usage:
      /auto [query]                - Execute AI agent task with natural language query
      /auto /help                  - Show this help message
      /auto /async /name <task_name> [query]         - Execute task asynchronously in background (name is required)
      /auto /queue [query]         - Add task to execution queue
      /auto /new [query]           - Create new conversation and execute query
      /auto /resume [id]           - Resume specific conversation by ID
      /auto /list                  - List all conversations
      /auto /rename [id] [name]    - Rename conversation
      /auto /export [id] [file_path] - Export conversation to markdown file
      /auto /command [command_file] [query] - Execute with specific command template

    Examples:
      /auto Add login functionality
      /auto /async /name my_task Refactor user module
      /auto /new Implement payment feature
      /auto /export conv-12345678 ./docs/conversation.md
      /auto /command deploy.md Deploy to production
    """


@byzerllm.prompt()
def auto_help_zh() -> str:
    """
    /auto 命令用法:
      /auto [查询]                 - 使用自然语言执行 AI 智能体任务
      /auto /help                  - 显示此帮助信息
      /auto /async /name <任务名> [查询]          - 在后台异步执行任务（必须提供 /name）
      /auto /queue [查询]          - 将任务添加到执行队列
      /auto /new [查询]            - 创建新对话并执行查询
      /auto /resume [id]           - 通过ID恢复特定对话
      /auto /list                  - 列出所有对话
      /auto /rename [id] [名称]    - 重命名对话
      /auto /export [id] [文件路径] - 将对话导出为markdown文件
      /auto /command [命令文件] [查询] - 使用特定命令模板执行

    示例:
      /auto 添加登录功能
      /auto /async /name my_task 重构用户模块
      /auto /new 实现支付功能
      /auto /export conv-12345678 ./docs/conversation.md
      /auto /command deploy.md 部署到生产环境
    """


@byzerllm.prompt()
def auto_help_ja() -> str:
    """
    /auto コマンド使用法:
      /auto [クエリ]               - 自然言語でAIエージェントタスクを実行
      /auto /help                  - このヘルプメッセージを表示
      /auto /async /name <タスク名> [クエリ]        - バックグラウンドで非同期にタスクを実行（/name 必須）
      /auto /queue [クエリ]        - タスクを実行キューに追加
      /auto /new [クエリ]          - 新しい会話を作成してクエリを実行
      /auto /resume [id]           - IDで特定の会話を再開
      /auto /list                  - すべての会話をリスト
      /auto /rename [id] [名前]    - 会話の名前を変更
      /auto /export [id] [ファイルパス] - 会話をmarkdownファイルにエクスポート
      /auto /command [コマンドファイル] [クエリ] - 特定のコマンドテンプレートで実行

    例:
      /auto ログイン機能を追加
      /auto /async /name my_task ユーザーモジュールをリファクタリング
      /auto /new 支払い機能を実装
      /auto /export conv-12345678 ./docs/conversation.md
      /auto /command deploy.md 本番環境にデプロイ
    """


@byzerllm.prompt()
def auto_help_ar() -> str:
    """
    استخدام أمر /auto:
      /auto [استعلام]              - تنفيذ مهمة وكيل AI باستخدام استعلام اللغة الطبيعية
      /auto /help                  - إظهار رسالة المساعدة هذه
      /auto /async /name <اسم_المهمة> [استعلام]       - تنفيذ المهمة بشكل غير متزامن في الخلفية (name مطلوب)
      /auto /queue [استعلام]       - إضافة مهمة إلى قائمة التنفيذ
      /auto /new [استعلام]         - إنشاء محادثة جديدة وتنفيذ الاستعلام
      /auto /resume [id]           - استئناف محادثة محددة بواسطة المعرف
      /auto /list                  - قائمة بجميع المحادثات
      /auto /rename [id] [اسم]     - إعادة تسمية المحادثة
      /auto /export [id] [مسار_الملف] - تصدير المحادثة إلى ملف markdown
      /auto /command [ملف_الأمر] [استعلام] - تنفيذ باستخدام قالب أمر محدد

    أمثلة:
      /auto إضافة وظيفة تسجيل الدخول
      /auto /async /name my_task إعادة هيكلة وحدة المستخدم
      /auto /new تنفيذ ميزة الدفع
      /auto /export conv-12345678 ./docs/conversation.md
      /auto /command deploy.md النشر في الإنتاج
    """


@byzerllm.prompt()
def auto_help_ru() -> str:
    """
    Использование команды /auto:
      /auto [запрос]               - Выполнить задачу AI агента с запросом на естественном языке
      /auto /help                  - Показать это сообщение справки
      /auto /async /name <имя_задачи> [запрос]        - Выполнить задачу асинхронно в фоне (требуется /name)
      /auto /queue [запрос]        - Добавить задачу в очередь выполнения
      /auto /new [запрос]          - Создать новый разговор и выполнить запрос
      /auto /resume [id]           - Возобновить конкретный разговор по ID
      /auto /list                  - Показать все разговоры
      /auto /rename [id] [имя]     - Переименовать разговор
      /auto /export [id] [путь_к_файлу] - Экспортировать разговор в markdown файл
      /auto /command [файл_команды] [запрос] - Выполнить с конкретным шаблоном команды

    Примеры:
      /auto Добавить функцию входа
      /auto /async /name my_task Рефакторить модуль пользователя
      /auto /new Реализовать функцию оплаты
      /auto /export conv-12345678 ./docs/conversation.md
      /auto /command deploy.md Развернуть в продакшен
    """


# /coding command help text definitions
@byzerllm.prompt()
def coding_help_en() -> str:
    """
    /coding command usage:
      /coding [query]       - Generate or modify code based on requirements
      /coding /help         - Show this help message
      /coding /apply [query] - Apply previous conversation context and generate code
      /coding /next [query] - Predict and suggest next development tasks

    Examples:
      /coding Add user authentication feature
      /coding /apply Implement password encryption
      /coding /next What should I do next?
    """


@byzerllm.prompt()
def coding_help_zh() -> str:
    """
    /coding 命令用法:
      /coding [查询]       - 根据需求生成或修改代码
      /coding /help        - 显示此帮助信息
      /coding /apply [查询] - 应用之前的对话上下文并生成代码
      /coding /next [查询] - 预测并建议下一步开发任务

    示例:
      /coding 添加用户认证功能
      /coding /apply 实现密码加密
      /coding /next 接下来应该做什么?
    """


@byzerllm.prompt()
def coding_help_ja() -> str:
    """
    /coding コマンド使用法:
      /coding [クエリ]       - 要件に基づいてコードを生成または変更
      /coding /help         - このヘルプメッセージを表示
      /coding /apply [クエリ] - 以前の会話コンテキストを適用してコードを生成
      /coding /next [クエリ] - 次の開発タスクを予測して提案

    例:
      /coding ユーザー認証機能を追加
      /coding /apply パスワード暗号化を実装
      /coding /next 次に何をすべきですか?
    """


@byzerllm.prompt()
def coding_help_ar() -> str:
    """
    استخدام أمر /coding:
      /coding [استعلام]       - إنشاء أو تعديل الكود بناءً على المتطلبات
      /coding /help         - إظهار رسالة المساعدة هذه
      /coding /apply [استعلام] - تطبيق سياق المحادثة السابقة وإنشاء الكود
      /coding /next [استعلام] - التنبؤ واقتراح مهام التطوير التالية

    أمثلة:
      /coding إضافة ميزة مصادقة المستخدم
      /coding /apply تنفيذ تشفير كلمة المرور
      /coding /next ماذا يجب أن أفعل بعد ذلك؟
    """


@byzerllm.prompt()
def coding_help_ru() -> str:
    """
    Использование команды /coding:
      /coding [запрос]       - Генерировать или изменять код на основе требований
      /coding /help         - Показать это сообщение справки
      /coding /apply [запрос] - Применить контекст предыдущего разговора и сгенерировать код
      /coding /next [запрос] - Предсказать и предложить следующие задачи разработки

    Примеры:
      /coding Добавить функцию аутентификации пользователя
      /coding /apply Реализовать шифрование пароля
      /coding /next Что мне делать дальше?
    """


# /chat command help text definitions
@byzerllm.prompt()
def chat_help_en() -> str:
    """
    /chat command usage:
      /chat [query]         - Chat with AI about code without modifying files
      /chat /help           - Show this help message
      /chat /new [query]    - Start new chat session
      /chat /learn [query]  - Learn from code and generate documentation
      /chat /review [query] - Review code changes
      /chat /no_context [query] - Chat without loading project files
      /chat /copy [query]   - Copy response to clipboard
      /chat /save [query]   - Save response to global memory
      /chat /mcp [query]    - Use MCP tools in chat
      /chat /rag [query]    - Use RAG search in chat

    Examples:
      /chat Explain the login logic
      /chat /new How does authentication work?
      /chat /learn Generate API documentation
      /chat /review Check recent changes
    """


@byzerllm.prompt()
def chat_help_zh() -> str:
    """
    /chat 命令用法:
      /chat [查询]         - 与AI聊天讨论代码，不修改文件
      /chat /help          - 显示此帮助信息
      /chat /new [查询]    - 开始新的聊天会话
      /chat /learn [查询]  - 从代码中学习并生成文档
      /chat /review [查询] - 审查代码变更
      /chat /no_context [查询] - 不加载项目文件的聊天
      /chat /copy [查询]   - 复制响应到剪贴板
      /chat /save [查询]   - 保存响应到全局记忆
      /chat /mcp [查询]    - 在聊天中使用MCP工具
      /chat /rag [查询]    - 在聊天中使用RAG搜索

    示例:
      /chat 解释登录逻辑
      /chat /new 认证是如何工作的?
      /chat /learn 生成API文档
      /chat /review 检查最近的变更
    """


@byzerllm.prompt()
def chat_help_ja() -> str:
    """
    /chat コマンド使用法:
      /chat [クエリ]         - ファイルを変更せずにAIとコードについてチャット
      /chat /help           - このヘルプメッセージを表示
      /chat /new [クエリ]    - 新しいチャットセッションを開始
      /chat /learn [クエリ]  - コードから学習してドキュメントを生成
      /chat /review [クエリ] - コード変更をレビュー
      /chat /no_context [クエリ] - プロジェクトファイルを読み込まずにチャット
      /chat /copy [クエリ]   - レスポンスをクリップボードにコピー
      /chat /save [クエリ]   - レスポンスをグローバルメモリに保存
      /chat /mcp [クエリ]    - チャットでMCPツールを使用
      /chat /rag [クエリ]    - チャットでRAG検索を使用

    例:
      /chat ログインロジックを説明
      /chat /new 認証はどのように機能しますか?
      /chat /learn APIドキュメントを生成
      /chat /review 最近の変更を確認
    """


@byzerllm.prompt()
def chat_help_ar() -> str:
    """
    استخدام أمر /chat:
      /chat [استعلام]         - الدردشة مع AI حول الكود دون تعديل الملفات
      /chat /help           - إظهار رسالة المساعدة هذه
      /chat /new [استعلام]    - بدء جلسة دردشة جديدة
      /chat /learn [استعلام]  - التعلم من الكود وإنشاء التوثيق
      /chat /review [استعلام] - مراجعة تغييرات الكود
      /chat /no_context [استعلام] - الدردشة دون تحميل ملفات المشروع
      /chat /copy [استعلام]   - نسخ الرد إلى الحافظة
      /chat /save [استعلام]   - حفظ الرد في الذاكرة العامة
      /chat /mcp [استعلام]    - استخدام أدوات MCP في الدردشة
      /chat /rag [استعلام]    - استخدام بحث RAG في الدردشة

    أمثلة:
      /chat اشرح منطق تسجيل الدخول
      /chat /new كيف تعمل المصادقة؟
      /chat /learn إنشاء توثيق API
      /chat /review التحقق من التغييرات الأخيرة
    """


@byzerllm.prompt()
def chat_help_ru() -> str:
    """
    Использование команды /chat:
      /chat [запрос]         - Чат с AI о коде без изменения файлов
      /chat /help           - Показать это сообщение справки
      /chat /new [запрос]    - Начать новую сессию чата
      /chat /learn [запрос]  - Изучить код и сгенерировать документацию
      /chat /review [запрос] - Проверить изменения кода
      /chat /no_context [запрос] - Чат без загрузки файлов проекта
      /chat /copy [запрос]   - Скопировать ответ в буфер обмена
      /chat /save [запрос]   - Сохранить ответ в глобальную память
      /chat /mcp [запрос]    - Использовать инструменты MCP в чате
      /chat /rag [запрос]    - Использовать поиск RAG в чате

    Примеры:
      /chat Объясните логику входа
      /chat /new Как работает аутентификация?
      /chat /learn Сгенерировать документацию API
      /chat /review Проверить недавние изменения
    """


# /lib command help text definitions
@byzerllm.prompt()
def lib_help_en() -> str:
    """
    /lib command usage:
      /lib /add <library>     - Add a library to the project
      /lib /remove <library>  - Remove a library from the project
      /lib /list              - List all added libraries
      /lib /list_all          - List all available libraries
      /lib /set-proxy [url]   - Set proxy for library downloads
      /lib /refresh           - Refresh the library repository
      /lib /get <package>     - Get documentation for a package
      /lib /help              - Show this help message

    Examples:
      /lib /add pandas
      /lib /remove numpy
      /lib /list
      /lib /get requests
    """


@byzerllm.prompt()
def lib_help_zh() -> str:
    """
    /lib 命令用法:
      /lib /add <库名>        - 向项目添加库
      /lib /remove <库名>     - 从项目中移除库
      /lib /list              - 列出所有已添加的库
      /lib /list_all          - 列出所有可用的库
      /lib /set-proxy [url]   - 设置库下载代理
      /lib /refresh           - 刷新库仓库
      /lib /get <包名>        - 获取包的文档
      /lib /help              - 显示此帮助信息

    示例:
      /lib /add pandas
      /lib /remove numpy
      /lib /list
      /lib /get requests
    """


@byzerllm.prompt()
def lib_help_ja() -> str:
    """
    /lib コマンド使用法:
      /lib /add <ライブラリ>   - プロジェクトにライブラリを追加
      /lib /remove <ライブラリ> - プロジェクトからライブラリを削除
      /lib /list              - 追加されたライブラリを一覧表示
      /lib /list_all          - 利用可能なライブラリを一覧表示
      /lib /set-proxy [url]   - ライブラリダウンロード用プロキシを設定
      /lib /refresh           - ライブラリリポジトリを更新
      /lib /get <パッケージ>   - パッケージのドキュメントを取得
      /lib /help              - このヘルプメッセージを表示

    例:
      /lib /add pandas
      /lib /remove numpy
      /lib /list
      /lib /get requests
    """


@byzerllm.prompt()
def lib_help_ar() -> str:
    """
    استخدام أمر /lib:
      /lib /add <مكتبة>       - إضافة مكتبة إلى المشروع
      /lib /remove <مكتبة>    - إزالة مكتبة من المشروع
      /lib /list              - عرض جميع المكتبات المضافة
      /lib /list_all          - عرض جميع المكتبات المتاحة
      /lib /set-proxy [url]   - تعيين وكيل لتحميل المكتبات
      /lib /refresh           - تحديث مستودع المكتبات
      /lib /get <حزمة>        - الحصول على وثائق الحزمة
      /lib /help              - إظهار رسالة المساعدة هذه

    أمثلة:
      /lib /add pandas
      /lib /remove numpy
      /lib /list
      /lib /get requests
    """


@byzerllm.prompt()
def lib_help_ru() -> str:
    """
    Использование команды /lib:
      /lib /add <библиотека>  - Добавить библиотеку в проект
      /lib /remove <библиотека> - Удалить библиотеку из проекта
      /lib /list              - Показать все добавленные библиотеки
      /lib /list_all          - Показать все доступные библиотеки
      /lib /set-proxy [url]   - Установить прокси для загрузки библиотек
      /lib /refresh           - Обновить репозиторий библиотек
      /lib /get <пакет>       - Получить документацию пакета
      /lib /help              - Показать это сообщение справки

    Примеры:
      /lib /add pandas
      /lib /remove numpy
      /lib /list
      /lib /get requests
    """


# /commit command help text definitions
@byzerllm.prompt()
def commit_help_en() -> str:
    """
    /commit command usage:
      /commit [message]     - Generate and commit changes with AI-generated commit message
      /commit /help          - Show this help message
      /commit /no_diff       - Commit without showing diff (future feature)

    Examples:
      /commit Fix user authentication bug
      /commit Add new feature for payment processing
      /commit Refactor database connection logic
    """


@byzerllm.prompt()
def commit_help_zh() -> str:
    """
    /commit 命令用法:
      /commit [消息]         - 使用AI生成的提交消息提交代码变更
      /commit /help          - 显示此帮助信息
      /commit /no_diff       - 不显示差异直接提交（未来功能）

    示例:
      /commit 修复用户认证bug
      /commit 添加支付处理新功能
      /commit 重构数据库连接逻辑
    """


@byzerllm.prompt()
def commit_help_ja() -> str:
    """
    /commit コマンド使用法:
      /commit [メッセージ]   - AI生成のコミットメッセージで変更をコミット
      /commit /help          - このヘルプメッセージを表示
      /commit /no_diff       - 差分を表示せずにコミット（将来の機能）

    例:
      /commit ユーザー認証バグを修正
      /commit 支払い処理の新機能を追加
      /commit データベース接続ロジックをリファクタリング
    """


@byzerllm.prompt()
def commit_help_ar() -> str:
    """
    استخدام أمر /commit:
      /commit [رسالة]       - إجراء commit للتغييرات مع رسالة commit مولدة بواسطة AI
      /commit /help          - إظهار رسالة المساعدة هذه
      /commit /no_diff       - إجراء commit دون إظهار الاختلافات (ميزة مستقبلية)

    أمثلة:
      /commit إصلاح خطأ مصادقة المستخدم
      /commit إضافة ميزة جديدة لمعالجة الدفع
      /commit إعادة هيكلة منطق اتصال قاعدة البيانات
    """


@byzerllm.prompt()
def commit_help_ru() -> str:
    """
    Использование команды /commit:
      /commit [сообщение]   - Зафиксировать изменения с сообщением коммита, сгенерированным AI
      /commit /help          - Показать это сообщение справки
      /commit /no_diff       - Зафиксировать без показа различий (будущая функция)

    Примеры:
      /commit Исправить ошибку аутентификации пользователя
      /commit Добавить новую функцию обработки платежей
      /commit Рефакторинг логики подключения к базе данных
    """


# /workflow command help text definitions
@byzerllm.prompt()
def workflow_help_en() -> str:
    """
    /workflow command usage:
      /workflow <workflow_name>              - Run a predefined workflow
      /workflow <workflow_name> key=value    - Run workflow with variable overrides
      /workflow /help                        - Show this help message

    Description:
      Execute predefined workflows with optional variable overrides via key-value pairs.

    Examples:
      /workflow my-workflow
      /workflow my-workflow query="Implement login"
      /workflow deploy env=prod version=1.0
      /workflow analysis task="Review code" depth=2

    Workflow search paths (by priority):
      1. .autocoderworkflow/                 (project-level, highest priority)
      2. .auto-coder/.autocoderworkflow/     (project-level)
      3. ~/.auto-coder/.autocoderworkflow/   (global-level)
    """


@byzerllm.prompt()
def workflow_help_zh() -> str:
    """
    /workflow 命令用法:
      /workflow <workflow名称>               - 运行预定义的workflow
      /workflow <workflow名称> key=value     - 运行workflow并覆盖变量
      /workflow /help                        - 显示此帮助信息

    说明:
      执行预定义的workflow，支持通过键值对参数覆盖workflow中的变量。

    示例:
      /workflow my-workflow
      /workflow my-workflow query="实现登录"
      /workflow deploy env=prod version=1.0
      /workflow analysis task="审查代码" depth=2

    Workflow 搜索路径（按优先级）:
      1. .autocoderworkflow/                 (项目级，最高优先级)
      2. .auto-coder/.autocoderworkflow/     (项目级)
      3. ~/.auto-coder/.autocoderworkflow/   (全局级)
    """


@byzerllm.prompt()
def workflow_help_ja() -> str:
    """
    /workflow コマンド使用法:
      /workflow <workflow名>                 - 事前定義されたworkflowを実行
      /workflow <workflow名> key=value       - 変数をオーバーライドしてworkflowを実行
      /workflow /help                        - このヘルプメッセージを表示

    説明:
      事前定義されたworkflowを実行し、キーと値のペアを使用して変数をオーバーライドできます。

    例:
      /workflow my-workflow
      /workflow my-workflow query="ログインを実装"
      /workflow deploy env=prod version=1.0
      /workflow analysis task="コードレビュー" depth=2

    Workflow 検索パス（優先順位順）:
      1. .autocoderworkflow/                 (プロジェクトレベル、最高優先度)
      2. .auto-coder/.autocoderworkflow/     (プロジェクトレベル)
      3. ~/.auto-coder/.autocoderworkflow/   (グローバルレベル)
    """


@byzerllm.prompt()
def workflow_help_ar() -> str:
    """
    استخدام أمر /workflow:
      /workflow <اسم_workflow>               - تشغيل workflow محدد مسبقاً
      /workflow <اسم_workflow> key=value     - تشغيل workflow مع تجاوز المتغيرات
      /workflow /help                        - إظهار رسالة المساعدة هذه

    الوصف:
      تنفيذ workflows المحددة مسبقاً مع إمكانية تجاوز المتغيرات عبر أزواج المفتاح والقيمة.

    أمثلة:
      /workflow my-workflow
      /workflow my-workflow query="تنفيذ تسجيل الدخول"
      /workflow deploy env=prod version=1.0
      /workflow analysis task="مراجعة الكود" depth=2

    مسارات البحث عن Workflow (حسب الأولوية):
      1. .autocoderworkflow/                 (مستوى المشروع، أعلى أولوية)
      2. .auto-coder/.autocoderworkflow/     (مستوى المشروع)
      3. ~/.auto-coder/.autocoderworkflow/   (مستوى عام)
    """


@byzerllm.prompt()
def workflow_help_ru() -> str:
    """
    Использование команды /workflow:
      /workflow <имя_workflow>               - Запустить предопределенный workflow
      /workflow <имя_workflow> key=value     - Запустить workflow с переопределением переменных
      /workflow /help                        - Показать это сообщение справки

    Описание:
      Выполнение предопределенных workflow с возможностью переопределения переменных через пары ключ-значение.

    Примеры:
      /workflow my-workflow
      /workflow my-workflow query="Реализовать вход"
      /workflow deploy env=prod version=1.0
      /workflow analysis task="Проверить код" depth=2

    Пути поиска Workflow (по приоритету):
      1. .autocoderworkflow/                 (уровень проекта, наивысший приоритет)
      2. .auto-coder/.autocoderworkflow/     (уровень проекта)
      3. ~/.auto-coder/.autocoderworkflow/   (глобальный уровень)
    """


# /async command help text definitions
@byzerllm.prompt()
def async_help_en() -> str:
    """
    /auto /async command usage:
      /auto /async /name <task_name> [query] - Execute task asynchronously in background (name is required)
      /auto /async /help                  - Show this help message
      /auto /async /list                  - List all async tasks
      /auto /async /task [task_id]        - Show details of a specific task
      /auto /async /kill <task_id>        - Kill a running task
      /auto /async /drop <task_id>        - Delete a task and all its related files

    Usage (name required):
      /auto /async /name <task_name> "<your query>"
      /auto /async /name <task_name> /model <model_name> "<your query>"
      /auto /async /name <task_name> /loop <count> "<your query>"
      /auto /async /name <task_name> /time <duration> "<your query>"
      /auto /async /name <task_name> /workflow <workflow_name> "<your query>"
      /auto /async /name <task_name> /vars '{"k":"v"}' "<your query>"

    Options:
      /name <task_name>   Required. Unique identifier for this async task.
      /model <model_name> Optional. Use a specific model.
      /loop <count>        Optional. Execute task multiple times.
      /time <duration>     Optional. Execute task for a specified duration (e.g., 5m, 1h).
      /workflow <name>     Optional. Execute a workflow by name.
      /vars <json>         Optional. JSON variables for the task.

    Examples:
      /auto /async /name my_task "Implement user authentication"
      /auto /async /name my_task /model gpt-4 "Refactor user module"
      /auto /async /name my_task /loop 3 "Continuous improvement task"
      /auto /async /name my_task /time 5m "Background analysis task"
      /auto /async /name my_task /workflow deploy "Deploy to staging"
      /auto /async /drop task123
    """


@byzerllm.prompt()
def async_help_zh() -> str:
    """
    /auto /async 命令用法:
      /auto /async /name <任务名> [查询] - 在后台异步执行任务（/name 为必选）
      /auto /async /help                  - 显示此帮助信息
      /auto /async /list                  - 列出所有异步任务
      /auto /async /task [任务ID]        - 显示特定任务的详情
      /auto /async /kill <任务ID>        - 终止正在运行的任务
      /auto /async /drop <任务ID>        - 删除任务及其所有相关文件

    用法（必须包含 /name）:
      /auto /async /name <任务名> "你的需求"
      /auto /async /name <任务名> /model <模型名> "你的需求"
      /auto /async /name <任务名> /loop <次数> "你的需求"
      /auto /async /name <任务名> /time <时长> "你的需求"
      /auto /async /name <任务名> /workflow <工作流名> "你的需求"
      /auto /async /name <任务名> /vars '{"k":"v"}' "你的需求"

    参数:
      /name <任务名>     必选。用于唯一标识该异步任务。
      /model <模型名>    可选。指定使用的模型。
      /loop <次数>       可选。多次执行任务。
      /time <时长>       可选。指定执行时长（如 5m, 1h）。
      /workflow <名称>   可选。按名称执行工作流。
      /vars <json>       可选。任务 JSON 变量。

    示例:
      /auto /async /name my_task "实现用户认证功能"
      /auto /async /name my_task /model gpt-4 "重构用户模块"
      /auto /async /name my_task /loop 3 "持续改进任务"
      /auto /async /name my_task /time 5m "后台分析任务"
      /auto /async /name my_task /workflow deploy "部署到预发环境"
      /auto /async /drop task123
    """


@byzerllm.prompt()
def async_help_ja() -> str:
    """
    /auto /async コマンド使用法:
      /auto /async /name <タスク名> [クエリ] - バックグラウンドで非同期にタスクを実行（/name は必須）
      /auto /async /help                  - このヘルプメッセージを表示
      /auto /async /list                  - すべての非同期タスクをリスト
      /auto /async /task [タスクID]      - 特定のタスクの詳細を表示
      /auto /async /kill <タスクID>      - 実行中のタスクを終了
      /auto /async /drop <タスクID>      - タスクとすべての関連ファイルを削除

    使用法（/name は必須）:
      /auto /async /name <タスク名> "クエリ"
      /auto /async /name <タスク名> /model <モデル名> "クエリ"
      /auto /async /name <タスク名> /loop <回数> "クエリ"
      /auto /async /name <タスク名> /time <時間> "クエリ"
      /auto /async /name <タスク名> /workflow <ワークフロー名> "クエリ"
      /auto /async /name <タスク名> /vars '{"k":"v"}' "クエリ"

    オプション:
      /name <タスク名>   必須。非同期タスクの一意の識別子。
      /model <モデル名>  任意。特定のモデルを使用。
      /loop <回数>       任意。タスクを複数回実行。
      /time <時間>       任意。指定した時間実行（例: 5m, 1h）。
      /workflow <名前>   任意。ワークフロー名で実行。
      /vars <json>       任意。タスクの JSON 変数。

    例:
      /auto /async /name my_task "ユーザー認証を実装"
      /auto /async /name my_task /model gpt-4 "ユーザーモジュールをリファクタリング"
      /auto /async /name my_task /loop 3 "継続的改善タスク"
      /auto /async /name my_task /time 5m "バックグラウンド分析タスク"
      /auto /async /name my_task /workflow deploy "ステージングにデプロイ"
      /auto /async /drop task123
    """


@byzerllm.prompt()
def async_help_ar() -> str:
    """
    استخدام أمر /auto /async:
      /auto /async /name <اسم_المهمة> [استعلام] - تنفيذ المهمة بشكل غير متزامن في الخلفية (name مطلوب)
      /auto /async /help                  - إظهار رسالة المساعدة هذه
      /auto /async /list                  - قائمة بجميع المهام غير المتزامنة
      /auto /async /task [معرف_المهمة]  - إظهار تفاصيل مهمة محددة
      /auto /async /kill <معرف_المهمة>  - إنهاء مهمة قيد التشغيل
      /auto /async /drop <معرف_المهمة>  - حذف المهمة وجميع الملفات ذات الصلة

    الاستخدام (name مطلوب):
      /auto /async /name <اسم_المهمة> "استعلامك"
      /auto /async /name <اسم_المهمة> /model <اسم_النموذج> "استعلامك"
      /auto /async /name <اسم_المهمة> /loop <عدد> "استعلامك"
      /auto /async /name <اسم_المهمة> /time <المدة> "استعلامك"
      /auto /async /name <اسم_المهمة> /workflow <اسم_workflow> "استعلامك"
      /auto /async /name <اسم_المهمة> /vars '{"k":"v"}' "استعلامك"

    الخيارات:
      /name <اسم_المهمة>   مطلوب. معرّف فريد للمهمة غير المتزامنة.
      /model <اسم_النموذج> اختياري. استخدام نموذج محدد.
      /loop <عدد>           اختياري. تنفيذ المهمة عدة مرات.
      /time <المدة>          اختياري. تنفيذ لمدة محددة (مثال: 5m, 1h).
      /workflow <الاسم>     اختياري. تنفيذ workflow بالاسم.
      /vars <json>          اختياري. متغيرات JSON للمهمة.

    أمثلة:
      /auto /async /name my_task "تنفيذ مصادقة المستخدم"
      /auto /async /name my_task /model gpt-4 "إعادة هيكلة وحدة المستخدم"
      /auto /async /name my_task /loop 3 "مهمة تحسين مستمرة"
      /auto /async /name my_task /time 5m "مهمة تحليل في الخلفية"
      /auto /async /name my_task /workflow deploy "النشر إلى بيئة staging"
      /auto /async /drop task123
    """


@byzerllm.prompt()
def async_help_ru() -> str:
    """
    Использование команды /auto /async:
      /auto /async /name <имя_задачи> [запрос] - Выполнить задачу асинхронно в фоне (требуется параметр /name)
      /auto /async /help                  - Показать это сообщение справки
      /auto /async /list                  - Показать все асинхронные задачи
      /auto /async /task [идентификатор] - Показать детали конкретной задачи
      /auto /async /kill <идентификатор> - Завершить выполняющуюся задачу
      /auto /async /drop <идентификатор> - Удалить задачу и все связанные с ней файлы

    Использование (требуется /name):
      /auto /async /name <имя_задачи> "<запрос>"
      /auto /async /name <имя_задачи> /model <имя_модели> "<запрос>"
      /auto /async /name <имя_задачи> /loop <количество> "<запрос>"
      /auto /async /name <имя_задачи> /time <продолжительность> "<запрос>"
      /auto /async /name <имя_задачи> /workflow <имя_workflow> "<запрос>"
      /auto /async /name <имя_задачи> /vars '{"k":"v"}' "<запрос>"

    Параметры:
      /name <имя_задачи>   Обязательно. Уникальный идентификатор асинхронной задачи.
      /model <имя_модели>  Необязательно. Использовать конкретную модель.
      /loop <количество>   Необязательно. Выполнить несколько раз.
      /time <продолжительность> Необязательно. Выполнять указанное время (напр., 5m, 1h).
      /workflow <имя>     Необязательно. Выполнить workflow по имени.
      /vars <json>         Необязательно. JSON-переменные задачи.

    Примеры:
      /auto /async /name my_task "Реализовать аутентификацию пользователя"
      /auto /async /name my_task /model gpt-4 "Рефакторинг модуля пользователя"
      /auto /async /name my_task /loop 3 "Задача непрерывного улучшения"
      /auto /async /name my_task /time 5m "Задача фонового анализа"
      /auto /async /name my_task /workflow deploy "Деплой на staging"
      /auto /async /drop task123
    """


# /rags command help text definitions
@byzerllm.prompt()
def rags_help_en() -> str:
    """
    /rags command usage - RAG knowledge base management

    Usage:
      /rags /list /local
          List all local knowledge bases

      /rags /local /path "<path>" /name "<name>" [/description "<description>"]
          Add a local knowledge base
          Note: Use quotes if path, name, or description contains spaces
          Example: /rags /local /path "/home/my docs" /name "my_docs" /description "My document library"

      /rags /local /remove "<name>"
          Remove a local knowledge base
          Example: /rags /local /remove "my_docs"
    """


@byzerllm.prompt()
def rags_help_zh() -> str:
    """
    /rags 命令帮助 - RAG 知识库管理

    用法:
      /rags /list /local
          列出所有本地知识库

      /rags /local /path "<路径>" /name "<名称>" [/description "<描述>"]
          添加本地知识库
          注意: 如果路径、名称或描述包含空格，需要用引号括起来
          示例: /rags /local /path "/home/my docs" /name "my_docs" /description "我的文档库"

      /rags /local /remove "<名称>"
          删除本地知识库
          示例: /rags /local /remove "my_docs"
    """


@byzerllm.prompt()
def rags_help_ja() -> str:
    """
    /rags コマンド使用法 - RAG ナレッジベース管理

    使用法:
      /rags /list /local
          すべてのローカルナレッジベースを一覧表示

      /rags /local /path "<パス>" /name "<名前>" [/description "<説明>"]
          ローカルナレッジベースを追加
          注意: パス、名前、説明にスペースが含まれる場合は引用符で囲んでください
          例: /rags /local /path "/home/my docs" /name "my_docs" /description "私のドキュメントライブラリ"

      /rags /local /remove "<名前>"
          ローカルナレッジベースを削除
          例: /rags /local /remove "my_docs"
    """


@byzerllm.prompt()
def rags_help_ar() -> str:
    """
    استخدام أمر /rags - إدارة قاعدة معرفة RAG

    الاستخدام:
      /rags /list /local
          عرض جميع قواعد المعرفة المحلية

      /rags /local /path "<المسار>" /name "<الاسم>" [/description "<الوصف>"]
          إضافة قاعدة معرفة محلية
          ملاحظة: استخدم علامات الاقتباس إذا كان المسار أو الاسم أو الوصف يحتوي على مسافات
          مثال: /rags /local /path "/home/my docs" /name "my_docs" /description "مكتبة المستندات الخاصة بي"

      /rags /local /remove "<الاسم>"
          حذف قاعدة معرفة محلية
          مثال: /rags /local /remove "my_docs"
    """


@byzerllm.prompt()
def rags_help_ru() -> str:
    """
    Использование команды /rags - Управление базой знаний RAG

    Использование:
      /rags /list /local
          Показать все локальные базы знаний

      /rags /local /path "<путь>" /name "<имя>" [/description "<описание>"]
          Добавить локальную базу знаний
          Примечание: Используйте кавычки, если путь, имя или описание содержат пробелы
          Пример: /rags /local /path "/home/my docs" /name "my_docs" /description "Моя библиотека документов"

      /rags /local /remove "<имя>"
          Удалить локальную базу знаний
          Пример: /rags /local /remove "my_docs"
    """


# Rags command runtime messages
RAGS_MESSAGES = {
    "rags_no_local_configs": {
        "en": "No local knowledge base configurations found",
        "zh": "暂无本地知识库配置",
        "ja": "ローカルナレッジベースの設定がありません",
        "ar": "لم يتم العثور على تكوينات قاعدة المعرفة المحلية",
        "ru": "Локальные базы знаний не найдены",
    },
    "rags_local_list_title": {
        "en": "Local Knowledge Base List:",
        "zh": "本地知识库列表:",
        "ja": "ローカルナレッジベース一覧:",
        "ar": "قائمة قواعد المعرفة المحلية:",
        "ru": "Список локальных баз знаний:",
    },
    "rags_name_label": {
        "en": "Name",
        "zh": "名称",
        "ja": "名前",
        "ar": "الاسم",
        "ru": "Имя",
    },
    "rags_path_label": {
        "en": "Path",
        "zh": "路径",
        "ja": "パス",
        "ar": "المسار",
        "ru": "Путь",
    },
    "rags_description_label": {
        "en": "Description",
        "zh": "描述",
        "ja": "説明",
        "ar": "الوصف",
        "ru": "Описание",
    },
    "rags_deleted_success": {
        "en": "Deleted local knowledge base: {{name}}",
        "zh": "已删除本地知识库: {{name}}",
        "ja": "ローカルナレッジベースを削除しました: {{name}}",
        "ar": "تم حذف قاعدة المعرفة المحلية: {{name}}",
        "ru": "Локальная база знаний удалена: {{name}}",
    },
    "rags_deleted_failed": {
        "en": "Deletion failed: Local knowledge base '{{name}}' not found",
        "zh": "删除失败: 未找到名为 '{{name}}' 的本地知识库",
        "ja": "削除に失敗しました: '{{name}}' という名前のローカルナレッジベースが見つかりません",
        "ar": "فشل الحذف: لم يتم العثور على قاعدة المعرفة المحلية '{{name}}'",
        "ru": "Ошибка удаления: Локальная база знаний '{{name}}' не найдена",
    },
    "rags_specify_name_to_delete": {
        "en": "Please specify the name of the knowledge base to delete",
        "zh": "请指定要删除的知识库名称",
        "ja": "削除するナレッジベースの名前を指定してください",
        "ar": "يرجى تحديد اسم قاعدة المعرفة المراد حذفها",
        "ru": "Пожалуйста, укажите имя базы знаний для удаления",
    },
    "rags_specify_path": {
        "en": "Please specify the knowledge base path using /path",
        "zh": "请使用 /path 指定知识库路径",
        "ja": "/path を使用してナレッジベースのパスを指定してください",
        "ar": "يرجى تحديد مسار قاعدة المعرفة باستخدام /path",
        "ru": "Пожалуйста, укажите путь к базе знаний с помощью /path",
    },
    "rags_specify_name": {
        "en": "Please specify the knowledge base name using /name",
        "zh": "请使用 /name 指定知识库名称",
        "ja": "/name を使用してナレッジベースの名前を指定してください",
        "ar": "يرجى تحديد اسم قاعدة المعرفة باستخدام /name",
        "ru": "Пожалуйста, укажите имя базы знаний с помощью /name",
    },
    "rags_added_success": {
        "en": "Added local knowledge base: {{name}}",
        "zh": "已添加本地知识库: {{name}}",
        "ja": "ローカルナレッジベースを追加しました: {{name}}",
        "ar": "تمت إضافة قاعدة المعرفة المحلية: {{name}}",
        "ru": "Локальная база знаний добавлена: {{name}}",
    },
    "rags_added_failed": {
        "en": "Addition failed: A knowledge base named '{{name}}' may already exist",
        "zh": "添加失败: 可能已存在同名知识库 '{{name}}'",
        "ja": "追加に失敗しました: '{{name}}' という名前のナレッジベースが既に存在する可能性があります",
        "ar": "فشلت الإضافة: قد تكون قاعدة المعرفة بالاسم '{{name}}' موجودة بالفعل",
        "ru": "Ошибка добавления: База знаний с именем '{{name}}' возможно уже существует",
    },
    "rags_error_occurred": {
        "en": "Operation failed: {{error}}",
        "zh": "操作失败: {{error}}",
        "ja": "操作に失敗しました: {{error}}",
        "ar": "فشلت العملية: {{error}}",
        "ru": "Операция не удалась: {{error}}",
    },
}


# Workflow command runtime messages
WORKFLOW_MESSAGES = {
    "workflow_error_no_name": {
        "en": "Error: Please specify workflow name",
        "zh": "错误: 请指定 workflow 名称",
        "ja": "エラー: workflow 名を指定してください",
        "ar": "خطأ: يرجى تحديد اسم workflow",
        "ru": "Ошибка: Укажите имя workflow",
    },
    "workflow_help_hint": {
        "en": "Use /workflow /help to see help information",
        "zh": "使用 /workflow /help 查看帮助信息",
        "ja": "/workflow /help を使用してヘルプ情報を表示",
        "ar": "استخدم /workflow /help لعرض معلومات المساعدة",
        "ru": "Используйте /workflow /help для просмотра справки",
    },
    "workflow_running": {
        "en": "Running workflow: {{workflow_name}}",
        "zh": "正在运行 workflow: {{workflow_name}}",
        "ja": "workflow を実行中: {{workflow_name}}",
        "ar": "تشغيل workflow: {{workflow_name}}",
        "ru": "Запуск workflow: {{workflow_name}}",
    },
    "workflow_parameters": {
        "en": "Parameters: {{kwargs}}",
        "zh": "参数: {{kwargs}}",
        "ja": "パラメータ: {{kwargs}}",
        "ar": "المعاملات: {{kwargs}}",
        "ru": "Параметры: {{kwargs}}",
    },
    "workflow_not_found": {
        "en": "Error: Workflow file not found: {{workflow_name}}",
        "zh": "错误: 未找到 workflow 文件: {{workflow_name}}",
        "ja": "エラー: workflow ファイルが見つかりません: {{workflow_name}}",
        "ar": "خطأ: لم يتم العثور على ملف workflow: {{workflow_name}}",
        "ru": "Ошибка: Файл workflow не найден: {{workflow_name}}",
    },
    "workflow_available_list": {
        "en": "Available workflows:",
        "zh": "可用的 workflows:",
        "ja": "利用可能な workflows:",
        "ar": "workflows المتاحة:",
        "ru": "Доступные workflows:",
    },
    "workflow_none_found": {
        "en": "(No workflows found)",
        "zh": "(没有找到任何 workflow)",
        "ja": "(workflow が見つかりません)",
        "ar": "(لم يتم العثور على أي workflows)",
        "ru": "(Workflows не найдены)",
    },
    "workflow_run_failed": {
        "en": "Failed to run workflow: {{error}}",
        "zh": "运行 workflow 失败: {{error}}",
        "ja": "workflow の実行に失敗しました: {{error}}",
        "ar": "فشل تشغيل workflow: {{error}}",
        "ru": "Не удалось запустить workflow: {{error}}",
    },
    "workflow_available_title": {
        "en": "Available workflows:",
        "zh": "可用的 workflows:",
        "ja": "利用可能な workflows:",
        "ar": "workflows المتاحة:",
        "ru": "Доступные workflows:",
    },
    "workflow_no_workflows_found": {
        "en": "No workflows found",
        "zh": "当前没有找到任何 workflow",
        "ja": "workflow が見つかりません",
        "ar": "لم يتم العثور على أي workflows",
        "ru": "Workflows не найдены",
    },
}


# Build the final message dictionary from prompt functions
# 使用 prompt() 获取 docstring 内容
COMMAND_HELP_MESSAGES = {
    "conf_help_text": {
        "en": conf_help_en.prompt(),
        "zh": conf_help_zh.prompt(),
        "ja": conf_help_ja.prompt(),
        "ar": conf_help_ar.prompt(),
        "ru": conf_help_ru.prompt(),
    },
    "auto_help_text": {
        "en": auto_help_en.prompt(),
        "zh": auto_help_zh.prompt(),
        "ja": auto_help_ja.prompt(),
        "ar": auto_help_ar.prompt(),
        "ru": auto_help_ru.prompt(),
    },
    "coding_help_text": {
        "en": coding_help_en.prompt(),
        "zh": coding_help_zh.prompt(),
        "ja": coding_help_ja.prompt(),
        "ar": coding_help_ar.prompt(),
        "ru": coding_help_ru.prompt(),
    },
    "chat_help_text": {
        "en": chat_help_en.prompt(),
        "zh": chat_help_zh.prompt(),
        "ja": chat_help_ja.prompt(),
        "ar": chat_help_ar.prompt(),
        "ru": chat_help_ru.prompt(),
    },
    "commit_help_text": {
        "en": commit_help_en.prompt(),
        "zh": commit_help_zh.prompt(),
        "ja": commit_help_ja.prompt(),
        "ar": commit_help_ar.prompt(),
        "ru": commit_help_ru.prompt(),
    },
    "lib_help_text": {
        "en": lib_help_en.prompt(),
        "zh": lib_help_zh.prompt(),
        "ja": lib_help_ja.prompt(),
        "ar": lib_help_ar.prompt(),
        "ru": lib_help_ru.prompt(),
    },
    "workflow_help_text": {
        "en": workflow_help_en.prompt(),
        "zh": workflow_help_zh.prompt(),
        "ja": workflow_help_ja.prompt(),
        "ar": workflow_help_ar.prompt(),
        "ru": workflow_help_ru.prompt(),
    },
    "async_help_text": {
        "en": async_help_en.prompt(),
        "zh": async_help_zh.prompt(),
        "ja": async_help_ja.prompt(),
        "ar": async_help_ar.prompt(),
        "ru": async_help_ru.prompt(),
    },
    "rags_help_text": {
        "en": rags_help_en.prompt(),
        "zh": rags_help_zh.prompt(),
        "ja": rags_help_ja.prompt(),
        "ar": rags_help_ar.prompt(),
        "ru": rags_help_ru.prompt(),
    },
}

# Merge workflow runtime messages into COMMAND_HELP_MESSAGES
COMMAND_HELP_MESSAGES.update(WORKFLOW_MESSAGES)

# Merge rags runtime messages into COMMAND_HELP_MESSAGES
COMMAND_HELP_MESSAGES.update(RAGS_MESSAGES)



"""
Workflow Agents 类型定义

定义 Subagent Workflow 系统中使用的数据类型和结构。
"""

from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional
from enum import Enum


@dataclass
class GlobalsConfig:
    """全局配置"""

    model: str = "v3_chat"
    product_mode: str = "lite"


@dataclass
class ConversationConfig:
    """会话共享策略配置"""

    default_action: str = "resume"  # resume | new | continue


@dataclass
class AttemptConfig:
    """AttemptCompletion 返回契约配置"""

    format: str = "json"  # json | text
    jsonpaths: Dict[str, str] = field(default_factory=dict)


@dataclass
class AgentSpec:
    """代理规格配置"""

    id: str
    path: str
    runner: str = "sdk"  # sdk | terminal
    model: Optional[str] = None


@dataclass
class RegexCondition:
    """正则条件配置"""

    input: str
    pattern: str
    flags: Optional[str] = None


@dataclass
class JsonPathCondition:
    """JSONPath 条件配置"""

    input: str
    path: str
    exists: Optional[bool] = None
    equals: Optional[Any] = None
    contains: Optional[str] = None


@dataclass
class TextCondition:
    """
    文本条件配置

    支持多种文本匹配方式：
    - contains: 包含指定字符串
    - not_contains: 不包含指定字符串
    - starts_with: 以指定字符串开头
    - ends_with: 以指定字符串结尾
    - equals: 完全相等
    - not_equals: 不相等
    - is_empty: 是否为空
    - matches: 正则表达式匹配
    - ignore_case: 是否忽略大小写（适用于 contains/starts_with/ends_with/equals）
    """

    input: str  # 输入模板，支持 ${...} 语法
    contains: Optional[str] = None
    not_contains: Optional[str] = None
    starts_with: Optional[str] = None
    ends_with: Optional[str] = None
    equals: Optional[str] = None
    not_equals: Optional[str] = None
    is_empty: Optional[bool] = None  # True: 检查为空, False: 检查非空
    matches: Optional[str] = None  # 正则表达式
    ignore_case: bool = False  # 是否忽略大小写


@dataclass
class WhenConfig:
    """条件判断配置"""

    regex: Optional[RegexCondition] = None
    jsonpath: Optional[JsonPathCondition] = None
    text: Optional[TextCondition] = None


@dataclass
class OutputConfig:
    """输出映射配置"""

    jsonpath: Optional[str] = None
    regex: Optional[str] = None
    regex_group: Optional[int] = None
    template: Optional[str] = None  # 模板字符串，如 "${attempt_result}"


@dataclass
class StepConversationConfig:
    """步骤级会话配置"""

    action: str = "resume"
    conversation_id: Optional[str] = None


@dataclass
class MergeConfig:
    """
    合并配置

    用于配置多副本执行时的结果合并行为。

    Attributes:
        when: 参与合并的条件（复用 WhenConfig）。
              只有满足条件的副本结果才会参与最终合并。
              条件中不需要指定 input，默认使用当前副本的 attempt_result。
    """

    when: Optional["WhenConfig"] = None


@dataclass
class StepSpec:
    """步骤规格配置"""

    id: str
    agent: str
    needs: List[str] = field(default_factory=list)
    with_args: Dict[str, Any] = field(default_factory=dict)
    when: Optional[WhenConfig] = None
    outputs: Dict[str, OutputConfig] = field(default_factory=dict)
    conversation: Optional[StepConversationConfig] = None
    replicas: int = 1  # 并行副本数，默认为1（不并行）
    merge: Optional[MergeConfig] = None  # 合并配置（用于多副本场景）


@dataclass
class MetadataConfig:
    """元数据配置"""

    name: str
    description: str = ""


@dataclass
class WorkflowSpec:
    """Workflow 总规格"""

    apiVersion: str
    kind: str
    metadata: MetadataConfig
    spec: "SpecConfig"


@dataclass
class SpecConfig:
    """Spec 配置"""

    globals: GlobalsConfig
    vars: Dict[str, Any] = field(default_factory=dict)
    conversation: ConversationConfig = field(default_factory=ConversationConfig)
    attempt: AttemptConfig = field(default_factory=AttemptConfig)
    agents: List[AgentSpec] = field(default_factory=list)
    steps: List[StepSpec] = field(default_factory=list)


# 执行结果相关类型


class StepStatus(str, Enum):
    """步骤执行状态"""

    SUCCESS = "success"
    FAILED = "failed"
    SKIPPED = "skipped"
    CANCELLED = "cancelled"


@dataclass
class StepResult:
    """步骤执行结果"""

    step_id: str
    status: StepStatus
    attempt_result: Optional[str] = None
    error: Optional[str] = None
    outputs: Dict[str, Any] = field(default_factory=dict)


@dataclass
class WorkflowResult:
    """工作流执行结果"""

    success: bool
    context: Dict[str, Any]
    step_results: List[StepResult]
    error: Optional[str] = None



import os
from autocoder.agent.base_agentic.tools.base_tool_resolver import BaseToolResolver
from autocoder.agent.base_agentic.types import ListFilesTool, ToolResult  # Import ToolResult from types
from typing import Optional, Dict, Any, List, Set, Union
import fnmatch
import re
import json
from loguru import logger
import typing
from autocoder.common import AutoCoderArgs

from autocoder.common.ignorefiles.ignore_file_utils import should_ignore

if typing.TYPE_CHECKING:
    from ..base_agent import BaseAgent


class ListFilesToolResolver(BaseToolResolver):
    def __init__(self, agent: Optional['BaseAgent'], tool: ListFilesTool, args: AutoCoderArgs):
        super().__init__(agent, tool, args)
        self.tool: ListFilesTool = tool  # For type hinting
        self.shadow_manager = self.agent.shadow_manager

    def list_files_in_dir(self, base_dir: str, recursive: bool, source_dir: str, is_outside_source: bool) -> Set[str]:
        """Helper function to list files in a directory"""
        result = set()
        try:
            if recursive:
                for root, dirs, files in os.walk(base_dir):
                    # Modify dirs in-place to skip ignored dirs early
                    dirs[:] = [d for d in dirs if not should_ignore(os.path.join(root, d))]
                    for name in files:
                        full_path = os.path.join(root, name)
                        if should_ignore(full_path):
                            continue
                        display_path = os.path.relpath(full_path, source_dir) if not is_outside_source else full_path
                        result.add(display_path)
                    for d in dirs:
                        full_path = os.path.join(root, d)
                        display_path = os.path.relpath(full_path, source_dir) if not is_outside_source else full_path
                        result.add(display_path + "/")
            else:
                for item in os.listdir(base_dir):
                    full_path = os.path.join(base_dir, item)
                    if should_ignore(full_path):
                        continue
                    display_path = os.path.relpath(full_path, source_dir) if not is_outside_source else full_path
                    if os.path.isdir(full_path):
                        result.add(display_path + "/")
                    else:
                        result.add(display_path)
        except Exception as e:
            logger.warning(f"Error listing files in {base_dir}: {e}")
        return result
        
    def list_files_with_shadow(self, list_path_str: str, recursive: bool, source_dir: str, absolute_source_dir: str, absolute_list_path: str) -> Union[ToolResult, List[str]]:
        """List files using shadow manager for path translation"""
        # Security check: Allow listing outside source_dir IF the original path is outside?
        is_outside_source = not absolute_list_path.startswith(absolute_source_dir)
        if is_outside_source:
            logger.warning(f"Listing path is outside the project source directory: {list_path_str}")

        # Check if shadow directory exists for this path
        shadow_exists = False
        shadow_dir_path = None
        if self.shadow_manager:
            try:
                shadow_dir_path = self.shadow_manager.to_shadow_path(absolute_list_path)
                if os.path.exists(shadow_dir_path) and os.path.isdir(shadow_dir_path):
                    shadow_exists = True
            except Exception as e:
                logger.warning(f"Error checking shadow path for {absolute_list_path}: {e}")

        # Validate that at least one of the directories exists
        if not os.path.exists(absolute_list_path) and not shadow_exists:
            return ToolResult(success=False, message=f"Error: Path not found: {list_path_str}")
        if os.path.exists(absolute_list_path) and not os.path.isdir(absolute_list_path):
            return ToolResult(success=False, message=f"Error: Path is not a directory: {list_path_str}")
        if shadow_exists and not os.path.isdir(shadow_dir_path):
            return ToolResult(success=False, message=f"Error: Shadow path is not a directory: {shadow_dir_path}")

        # Collect files from shadow and/or source directory
        shadow_files_set = set()
        if shadow_exists:
            shadow_files_set = self.list_files_in_dir(shadow_dir_path, recursive, source_dir, is_outside_source)

        source_files_set = set()
        if os.path.exists(absolute_list_path) and os.path.isdir(absolute_list_path):
            source_files_set = self.list_files_in_dir(absolute_list_path, recursive, source_dir, is_outside_source)

        # Merge results, prioritizing shadow files if exist
        if shadow_exists:
            merged_files = shadow_files_set.union(
                {f for f in source_files_set if f not in shadow_files_set}
            )
        else:
            merged_files = source_files_set

        try:
            message = f"Successfully listed contents of '{list_path_str}' (Recursive: {recursive}). Found {len(merged_files)} items."
            logger.info(message)
            return sorted(merged_files)
        except Exception as e:
            logger.error(f"Error listing files in '{list_path_str}': {str(e)}")
            return ToolResult(success=False, message=f"An unexpected error occurred while listing files: {str(e)}")

    def list_files_normal(self, list_path_str: str, recursive: bool, source_dir: str, absolute_source_dir: str, absolute_list_path: str) -> Union[ToolResult, List[str]]:
        """List files directly without using shadow manager"""
        # Security check: Allow listing outside source_dir IF the original path is outside?
        is_outside_source = not absolute_list_path.startswith(absolute_source_dir)
        if is_outside_source:
            logger.warning(f"Listing path is outside the project source directory: {list_path_str}")

        # Validate that the directory exists
        if not os.path.exists(absolute_list_path):
            return ToolResult(success=False, message=f"Error: Path not found: {list_path_str}")
        if not os.path.isdir(absolute_list_path):
            return ToolResult(success=False, message=f"Error: Path is not a directory: {list_path_str}")

        # Collect files from the directory
        files_set = self.list_files_in_dir(absolute_list_path, recursive, source_dir, is_outside_source)

        try:
            message = f"Successfully listed contents of '{list_path_str}' (Recursive: {recursive}). Found {len(files_set)} items."
            logger.info(message)
            return sorted(files_set)
        except Exception as e:
            logger.error(f"Error listing files in '{list_path_str}': {str(e)}")
            return ToolResult(success=False, message=f"An unexpected error occurred while listing files: {str(e)}")

    def resolve(self) -> ToolResult:
        """Resolve the list files tool by calling the appropriate implementation"""
        list_path_str = self.tool.path
        recursive = self.tool.recursive or False
        source_dir = self.args.source_dir or "."
        absolute_source_dir = os.path.abspath(source_dir)
        absolute_list_path = os.path.abspath(os.path.join(source_dir, list_path_str))

        # Choose the appropriate implementation based on whether shadow_manager is available
        if self.shadow_manager:
            result = self.list_files_with_shadow(list_path_str, recursive, source_dir, absolute_source_dir, absolute_list_path)
        else:
            result = self.list_files_normal(list_path_str, recursive, source_dir, absolute_source_dir, absolute_list_path)

        # Handle the case where the implementation returns a sorted list instead of a ToolResult
        if isinstance(result, list):
            message = f"Successfully listed contents of '{list_path_str}' (Recursive: {recursive}). Found {len(result)} items."
            return ToolResult(success=True, message=message, content=result)
        else:
            return result



import os
import traceback
import re
from PIL import Image

try:
    from paddleocr import PaddleOCR
except ImportError:
    PaddleOCR = None

try:
    import paddlex as paddlex_module
except ImportError:
    paddlex_module = None

import byzerllm
from byzerllm.utils.client import code_utils
from autocoder.utils.llms import get_single_llm
from loguru import logger
from typing import List, Tuple, Optional
from autocoder.common.text_similarity import TextSimilarity
from pydantic import BaseModel


class ReplaceInFileTool(BaseModel):
    path: str
    diff: str


class ImageLoader:
    """
    A class for loading and processing images, extracting text and tables from them,
    and converting the content to markdown format.
    """
    
    # 存储不同参数组合的PaddleOCR实例
    _ocr_instances = {}
    
    @staticmethod
    def parse_diff(diff_content: str) -> List[Tuple[str, str]]:
        """
        Parses the diff content into a list of (search_block, replace_block) tuples.
        """
        blocks = []
        lines = diff_content.splitlines(keepends=True)
        i = 0
        n = len(lines)

        while i < n:
            line = lines[i]
            if line.strip() == "<<<<<<< SEARCH":
                i += 1
                search_lines = []
                # Accumulate search block
                while i < n and lines[i].strip() != "=======":
                    search_lines.append(lines[i])
                    i += 1
                if i >= n:
                    logger.warning("Unterminated SEARCH block found in diff content.")
                    break
                i += 1  # skip '======='
                replace_lines = []
                # Accumulate replace block
                while i < n and lines[i].strip() != ">>>>>>> REPLACE":
                    replace_lines.append(lines[i])
                    i += 1
                if i >= n:
                    logger.warning("Unterminated REPLACE block found in diff content.")
                    break
                i += 1  # skip '>>>>>>> REPLACE'

                search_block = ''.join(search_lines)
                replace_block = ''.join(replace_lines)
                blocks.append((search_block, replace_block))
            else:
                i += 1

        if not blocks and diff_content.strip():
            logger.warning(f"Could not parse any SEARCH/REPLACE blocks from diff: {diff_content}")
        return blocks
    
    @staticmethod
    def paddleocr_extract_text(
        file_path,
        lang='ch',
        use_angle_cls=True,
        page_num=10,
        slice_params=None,
        det_model_dir=None,
        rec_model_dir=None,
        **kwargs
    ):
        """
        使用 PaddleOCR 识别文本，支持图片、PDF、超大图像滑动窗口

        Args:
            file_path: 图片或PDF路径
            lang: 语言，默认中文
            use_angle_cls: 是否启用方向分类
            page_num: 识别PDF时的最大页数
            slice_params: 超大图像滑动窗口参数 dict
            det_model_dir: 自定义检测模型路径
            rec_model_dir: 自定义识别模型路径
            kwargs: 其他paddleocr参数
        Returns:
            识别出的纯文本字符串
        """
        if PaddleOCR is None:
            print("paddleocr not installed")
            return ""

        # 创建一个参数的哈希键，用于在缓存中存储OCR实例
        param_key = f"{lang}_{use_angle_cls}_{page_num}_{det_model_dir}_{rec_model_dir}_{hash(frozenset(kwargs.items()) if kwargs else 0)}"
        
        # 检查是否已经有对应参数的OCR实例
        if param_key not in ImageLoader._ocr_instances:
            try:
                # 初始化OCR并缓存
                ImageLoader._ocr_instances[param_key] = PaddleOCR(
                    use_angle_cls=use_angle_cls,
                    lang=lang,
                    page_num=page_num,
                    det_model_dir=det_model_dir,
                    rec_model_dir=rec_model_dir,
                    **kwargs
                )
                logger.info(f"初始化新的PaddleOCR实例，参数：{param_key}")
            except Exception:
                traceback.print_exc()
                return ""
        
        # 使用缓存的OCR实例
        ocr = ImageLoader._ocr_instances[param_key]

        try:
            ext = os.path.splitext(file_path)[1].lower()

            # 处理PDF
            if ext == ".pdf":
                result = ocr.ocr(file_path, cls=True) # result is list of pages, each page is list of lines
                lines = []
                if result and isinstance(result, list):
                    for page in result:
                        if page and isinstance(page, list):
                            for line_info in page: # line_info is [points, (text, confidence)]
                                try:
                                    # Check structure: [points, (text, confidence)]
                                    if isinstance(line_info, (list, tuple)) and len(line_info) == 2 and \
                                       isinstance(line_info[1], (list, tuple)) and len(line_info[1]) >= 1:
                                        txt = line_info[1][0]
                                        if isinstance(txt, str):
                                            lines.append(txt)
                                        else:
                                            logger.warning(f"Extracted text is not a string in PDF: {txt} (type: {type(txt)}). Skipping.")
                                    else:
                                        logger.warning(f"Unexpected line_info structure in PDF: {line_info}. Skipping.")
                                except Exception as e:
                                    logger.warning(f"Error processing line_info in PDF: {line_info}. Error: {e}")
                return "\n".join(lines)

            # 处理图片
            else: # Image processing
                if slice_params is not None:
                    result = ocr.ocr(file_path, cls=True, slice=slice_params)
                else:
                    result = ocr.ocr(file_path, cls=True) # result is [[[points, (text, confidence)], ...]] for single image

                lines = []
                # Standardize handling: PaddleOCR often returns a list containing one item for single images.
                # result = [page_result] where page_result = [[line1_info], [line2_info], ...]
                if result and isinstance(result, list):
                     # Heuristic: Treat 'result' as the list of pages directly.
                     # This handles both single image wrapped in list and multi-page PDFs consistently.
                     page_list = result

                     for page in page_list:
                        if page and isinstance(page, list):
                            for line_info in page: # line_info is [points, (text, confidence)]
                                try:
                                    # Check structure: [points, (text, confidence)]
                                    if isinstance(line_info, (list, tuple)) and len(line_info) == 2 and \
                                       isinstance(line_info[1], (list, tuple)) and len(line_info[1]) >= 1:
                                        txt = line_info[1][0]
                                        if isinstance(txt, str):
                                            lines.append(txt)
                                        else:
                                            # Handle potential nested lists in text: join them? Or log?
                                            if isinstance(txt, list):
                                                processed_txt = " ".join(map(str, txt))
                                                logger.warning(f"Extracted text is a list in Image: {txt}. Joined as: '{processed_txt}'.")
                                                lines.append(processed_txt) # Attempt to join if it's a list of strings/convertibles
                                            else:
                                                logger.warning(f"Extracted text is not a string in Image: {txt} (type: {type(txt)}). Skipping.")
                                    else:
                                        logger.warning(f"Unexpected line_info structure in Image: {line_info}. Skipping.")
                                except Exception as e:
                                    logger.warning(f"Error processing line_info in Image: {line_info}. Error: {e}")
                return "\n".join(lines)
        except Exception:
            traceback.print_exc()
            return ""
    
    @staticmethod
    def paddlex_table_extract_markdown(image_path):
        """
        使用 PaddleX 表格识别pipeline，抽取表格并转换为markdown格式

        Args:
            image_path: 图片路径
        Returns:
            markdown格式的表格字符串
        """
        if paddlex_module is None:
            print("paddlex not installed")
            return ""

        try:
            # 创建 pipeline
            pipeline = paddlex_module.create_pipeline(pipeline='table_recognition')
            # 预测
            outputs = pipeline.predict([image_path])
            if not outputs:
                return ""

            md_results = []
            for res in outputs:
                # 获取HTML表格
                html = None
                try:
                    html = res.to_html() if hasattr(res, "to_html") else None
                except Exception:
                    html = None

                # 如果没有to_html方法，尝试res.print()内容中提取，或跳过
                if html is None:
                    try:
                        from io import StringIO
                        import sys
                        buffer = StringIO()
                        sys_stdout = sys.stdout
                        sys.stdout = buffer
                        res.print()
                        sys.stdout = sys_stdout
                        html = buffer.getvalue()
                    except Exception:
                        html = ""

                # 转markdown
                md = ImageLoader.html_table_to_markdown(html)
                md_results.append(md)

            return "\n\n".join(md_results)
        except Exception:
            traceback.print_exc()
            return ""
    
    @staticmethod
    def html_table_to_markdown(html):
        """
        简单将HTML table转换为markdown table
        """
        try:
            from bs4 import BeautifulSoup
        except ImportError:
            print("BeautifulSoup4 not installed, cannot convert HTML to markdown")
            return ""

        try:
            soup = BeautifulSoup(html, "html.parser")
            table = soup.find("table")
            if table is None:
                return ""

            rows = []
            for tr in table.find_all("tr"):
                cells = tr.find_all(["td", "th"])
                row = [cell.get_text(strip=True) for cell in cells]
                rows.append(row)

            if not rows:
                return ""

            # 生成markdown
            md_lines = []
            header = rows[0]
            md_lines.append("| " + " | ".join(header) + " |")
            md_lines.append("|" + "|".join(["---"] * len(header)) + "|")

            for row in rows[1:]:
                md_lines.append("| " + " | ".join(row) + " |")

            return "\n".join(md_lines)
        except Exception:
            traceback.print_exc()
            return ""

    @staticmethod
    def extract_replace_in_file_tools(response)->List[ReplaceInFileTool]:
        tools = []
        # Pattern to match replace_in_file tool blocks
        pattern = r'<replace_in_file>\s*<path>(.*?)</path>\s*<diff>(.*?)</diff>\s*</replace_in_file>'
        matches = re.finditer(pattern, response, re.DOTALL)
        
        for match in matches:
            path = match.group(1).strip()
            diff = match.group(2).strip()
            tools.append(ReplaceInFileTool(path=path, diff=diff))
        
        return tools        
    
    @staticmethod
    def format_table_in_content(content: str, llm=None) -> str:
        """Format table content from OCR results into markdown format.
        
        Args:
            content: The OCR text content that may contain tables
            llm: The language model to use for formatting
            
        Returns:
            Formatted content with tables converted to markdown
        """    
            
        @byzerllm.prompt()
        def _format_table(content: str)->str:
            '''
            # 表格格式化任务
            
            你是一个专业的OCR后处理专家，擅长将OCR识别出的表格数据转换为规范的Markdown表格。
            
            ## 输入内容分析
            
            OCR识别的表格通常会有以下特点：
            1. 每个单元格可能被识别为单独的一行
            2. 表格的行列结构可能不明显
            3. 可能包含非表格的文本内容
            4. 可能存在多个表格
            
            ## 你的任务
            
            1. 识别内容中的表格数据
            2. 将表格数据转换为标准Markdown格式
            3. 保留非表格的文本内容
            4. 使用replace_in_file工具格式输出结果
            
            ## 输出格式
            
            必须使用以下格式输出结果：
            
            ```
            <replace_in_file>
            <path>content</path>
            <diff>
            <<<<<<< SEARCH
            [原始表格文本，精确匹配]
            =======
            [转换后的Markdown表格]
            >>>>>>> REPLACE
            </diff>
            </replace_in_file>
            ```
            
            ## 示例
            
            原始OCR文本：
            ```
            下面是库存情况：
            产品名称 
            价格 
            库存
            苹果手机 
            8999 352
            华为平板 
            4599 
            128
            小米电视 
            3299 
            89
            可以看到在，整体库存和价格是健康的。
            ```
            
            转换后的输出：
            ```
            <replace_in_file>
            <path>content</path>
            <diff>
            <<<<<<< SEARCH
            产品名称 
            价格 
            库存
            苹果手机 
            8999 352
            华为平板 
            4599 
            128
            小米电视 
            3299 
            89
            =======
            | 产品名称 | 价格 | 库存 |
            |---------|------|------|
            | 苹果手机 | 8999 | 352 |
            | 华为平板 | 4599 | 128 |
            | 小米电视 | 3299 | 89  |
            >>>>>>> REPLACE
            </diff>
            </replace_in_file>
            ```
            
            ## 处理规则
            
            1. 表格识别：
               - 分析行列结构，识别表头和数据行
               - 如果一行中有多个值，可能是一行表格数据
               - 连续的短行可能是表格的单元格
            
            2. Markdown格式：
               - 表头行使用`|`分隔各列
               - 在表头下方添加分隔行`|---|---|---|`
               - 对齐各列数据
               - 保持原始数据的完整性
            
            3. 多表格处理：
               - 为每个表格创建单独的replace_in_file块
               - 保持表格在原文中的相对位置
            
            4. 非表格内容：
               - 保留原始格式
               - 不要修改非表格文本
            
            ## 处理以下内容
            
            {{content}}
            '''
        
        # Run the prompt with the provided content
        tool_response = _format_table.with_llm(llm).run(content)                    
        
        # Extract tools from the response
        tools = ImageLoader.extract_replace_in_file_tools(tool_response)    
        
        # Process each tool to apply the replacements
        formatted_content = content
        for tool in tools:
            # For in-memory content replacement (not actual file modification)            
            # Parse the diff to get search/replace blocks
            blocks = ImageLoader.parse_diff(tool.diff)
            # Apply each replacement to the content                
            for search_block, replace_block in blocks:
                # Check if the search_block exists in the content
                if search_block in formatted_content:
                    # Replace and verify the replacement occurred
                    new_content = formatted_content.replace(search_block, replace_block)
                    if new_content == formatted_content:
                        logger.warning(f"Replacement failed despite search block found. Search block length: {len(search_block)}")
                        print(f"\n=== FAILED SEARCH BLOCK ===\n{search_block}\n=== END FAILED SEARCH BLOCK ===\n")
                    formatted_content = new_content
                else:
                    # Fallback to similarity matching when exact match fails
                    logger.warning(f"Search block not found in content. Trying similarity matching. Search block length: {len(search_block)}")
                    print(f"\n=== NOT FOUND SEARCH BLOCK (trying similarity) ===\n{search_block}\n=== END NOT FOUND SEARCH BLOCK ===\n")
                    
                    # Use TextSimilarity to find the best matching window
                    similarity, best_window = TextSimilarity(search_block, formatted_content).get_best_matching_window()
                    similarity_threshold = 0.8  # Can be adjusted based on needs
                    
                    if similarity > similarity_threshold:
                        logger.info(f"Found similar block with similarity {similarity:.2f}")
                        print(f"\n=== SIMILAR BLOCK FOUND (similarity: {similarity:.2f}) ===\n{best_window}\n=== END SIMILAR BLOCK ===\n")
                        formatted_content = formatted_content.replace(best_window, replace_block, 1)
                    else:
                        logger.warning(f"No similar block found. Best similarity: {similarity:.2f}")
        
        return formatted_content
    
    @staticmethod
    def extract_text_from_image(
        image_path: str,
        llm,
        engine: str = "vl",
        product_mode: str = "lite",
        paddle_kwargs: dict = None
    ) -> str:
        """
        识别图片或PDF中的所有文本内容，包括表格（以markdown table格式）

        Args:
            image_path: 图片或PDF路径
            llm: LLM对象或字符串（模型名）
            engine: 选择识别引擎
                - "vl": 视觉语言模型
                - "paddle": PaddleOCR
                - "paddle_table": PaddleX表格识别
            product_mode: get_single_llm的参数
            paddle_kwargs: dict，传递给PaddleOCR的参数
        Returns:
            markdown内容字符串
        """
        if isinstance(llm, str):
            llm = get_single_llm(llm, product_mode=product_mode)

        markdown_content = ""

        if engine == "vl":
            try:
                vl_model = llm.get_sub_client("vl_model") if llm.get_sub_client("vl_model") else llm

                @byzerllm.prompt()
                def analyze_image(image_path):
                    """
                    {{ image }}
                    你是一名图像理解专家，请识别这张图片中的所有内容，优先识别文字和表格。
                    对于普通文字，输出为段落文本。
                    对于表格截图，转换成markdown table格式输出。
                    请根据内容顺序，整合成一份markdown文档。
                    只返回markdown内容，不要添加额外解释。
                    """
                    image = byzerllm.Image.load_image_from_path(image_path)
                    return {"image": image}

                result = analyze_image.with_llm(vl_model).run(image_path)
                md_blocks = code_utils.extract_code(result, language="markdown")
                if md_blocks:
                    markdown_content = md_blocks[-1][1]
                else:
                    markdown_content = result.strip()
                if not markdown_content:
                    raise ValueError("Empty markdown from vl_model")
                return markdown_content

            except Exception:
                traceback.print_exc()
                return ""

        elif engine == "paddle":
            if paddle_kwargs is None:
                paddle_kwargs = {}

            markdown_content = ImageLoader.paddleocr_extract_text(image_path, **paddle_kwargs)
            return markdown_content

        elif engine == "paddle_table":
            markdown_content = ImageLoader.paddlex_table_extract_markdown(image_path)
            return markdown_content

        else:
            print(f"Unknown engine type: {engine}. Supported engines are 'vl', 'paddle', and 'paddle_table'.")
            return ""
    
    @staticmethod
    def image_to_markdown(
        image_path: str,
        llm,
        engine: str = "paddle",
        product_mode: str = "lite",
        paddle_kwargs: dict = None
    ) -> str:
        """
        识别图片或PDF内容，生成markdown文件

        Args:
            image_path: 文件路径
            llm: LLM对象或字符串
            engine: 'vl'、'paddle'或'paddle_table'
            product_mode: LLM参数
            paddle_kwargs: dict，传递给PaddleOCR参数
        Returns:
            markdown内容字符串
        """
        logger.info(f"image_path: {image_path} engine: {engine} product_mode: {product_mode} paddle_kwargs: {paddle_kwargs}")        

        # 新增：如果 engine 为 paddle 且 PaddleOCR 为 None，直接返回空字符串
        if engine == "paddle" and PaddleOCR is None:
            logger.warning("PaddleOCR 未安装，无法识别图片内容，直接返回空字符串。")
            return ""

        md_content = ImageLoader.extract_text_from_image(
            image_path,
            llm,
            engine=engine,
            product_mode=product_mode,
            paddle_kwargs=paddle_kwargs
        )
            
        # Get directory and filename separately
        dir_name = os.path.dirname(image_path)
        file_name = os.path.basename(image_path)
        base_name = os.path.splitext(file_name)[0]
        # Create new path with dot before filename
        md_path = os.path.join(dir_name, f".{base_name}.md")
        try:
            with open(md_path, "w", encoding="utf-8") as f:
                f.write(md_content)
        except Exception:
            traceback.print_exc()

        return md_content

